import unittest
import tethne.readers as rd
from tethne.data import Paper
import os
import numpy as np

class TestLoad(unittest.TestCase):

    def setUp(self):
        td_path = "./testin/mallet/top_doc"
        wt_path = "./testin/mallet/word_top"
        tk_path = "./testin/mallet/topic_keys"
        m_path = "./testin/mallet/metadata"
        
        Z = 100
        
        self.L = rd.mallet.load(td_path, wt_path, tk_path, Z, m_path)
                
    def test_top_doc(self):
        """
        Each row of L.doc_topic should sum to ~ 1.
        """
        
        D = self.L.doc_topic.shape[0]
        Z = self.L.doc_topic.shape[1]
        
        for d in xrange(D):
            self.assertEqual(1.0, np.around(np.sum(self.L.doc_topic[d,:])))
    
    def test_word_top(self):
        """
        Each row of L.top_word should sum to ~ 1.
        """        
        Z = self.L.top_word.shape[0]
        W = self.L.top_word.shape[1]
        
        for z in xrange(Z):
            self.assertEqual(1.0, np.around(np.sum(self.L.top_word[z,:])))
    
    def test_topic_keys(self):
        """
        There should be a list of words for each topic.
        """
        Z = self.L.doc_topic.shape[1]
        
        self.assertEqual(len(self.L.top_keys), Z)
        
    def tearDown(self):
        pass

class TestRead(unittest.TestCase):

    def setUp(self):
        td_path = "./testin/mallet/top_doc"
        wt_path = "./testin/mallet/word_top"
        tk_path = "./testin/mallet/topic_keys"
        m_path = "./testin/mallet/metadata"
        
        Z = 100

        self.L = rd.mallet.load(td_path, wt_path, tk_path, Z, m_path)        
        self.papers = rd.mallet.read(td_path, wt_path, tk_path, Z, m_path)
    
    def test_num_papers(self):
        """
        Each document in the LDAModel should yield a :class:`.Paper`\.
        """
        
        self.assertEqual(self.L.doc_topic.shape[0], len(self.papers))
    
    def test_papers_content(self):
        """
        Should return :class:`.Paper` objects.
        """
        
        self.assertIsInstance(self.papers[0], Paper)

if __name__ == '__main__':
    unittest.main()



"""
Methods for working with PubMed data are still under development. Please use
with care.

.. autosummary::

   read

"""

import tethne.data as ds
import xml.etree.ElementTree as ET
import uuid


# PubMed functions
def _pubmed_file_id(filename):
    """Future (not implemented).

    Given a filename (presumed to contain PubMed compatible IDs)
    return an xml string for each article associated with that ID.

    Parameters
    ----------
    filename : string
        Path to a file containing PubMed-compatible IDs.

    Returns
    -------
    list : list
        A list of XML strings.

    """

    return None

def read(filepath):
    """
    Given a file with PubMed XML, return a list of :class:`.Paper` instances.

    See the following hyperlinks regarding possible structures of XML:
    * http://www.ncbi.nlm.nih.gov/pmc/pmcdoc/tagging-guidelines/citations/v2/citationtags.html#2Articlewithmorethan10authors%28listthefirst10andaddetal%29
    * http://dtd.nlm.nih.gov/publishing/

    Each :class:`.Paper` is tagged with an accession id for this
    read/conversion.

    **Usage**

    .. code-block:: python

       >>> import tethne.readers as rd
       >>> papers = rd.pubmed.read("/Path/to/PubMedData.xml")

    Parameters
    ----------
    filepath : string
        Path to PubMed XML file.

    Returns
    -------
    meta_list : list
        A list of :class:`.Paper` instances.
    """

    try:
        with open(filepath,'r') as f:
            tree = ET.fromstring(text, parser)(filepath)
            root = tree.getroot()

    except IOError: # File does not exist, or couldn't be read.
        raise IOError("File does not exist, or cannot be read.")

    accession = str(uuid.uuid4())

    # define location of simple article meta data relative to xml tree rooted
    # at 'article'
    meta_loc = {'atitle':'./front/article-meta/title-group/article-title',
                'jtitle':('./front/journal-meta/journal-title-group/' +
                          'journal-title'),
                'volume':'./front/article-meta/volume',
                'issue':'./front/article-meta/issue',
                'spage':'./front/article-meta/fpage',
                'epage':'./front/article-meta/lpage'}

    # location relative to element-citation element
    cit_meta_loc = {'atitle':'./article-title',
                    'jtitle':'./source',
                    'date':'./year',
                    'volume':'./volume',
                    'spage':'./fpage',
                    'epage':'./epage'}

    meta_list = []
    for article in root.iter('article'):
        paper = ds.Paper()

        # collect information from the 'front' section of the article
        # collect the simple data
        for key in meta_loc.iterkeys():
            key_data = article.find(meta_loc[key])
            if key_data is not None:
                paper[key] = key_data.text
            else:
                paper[key] = None

        # collect doi and pmid
        id_list = article.findall('./front/article-meta/article-id')
        for identifier in id_list:
            id_type = identifier.get('pub-id-type')
            if id_type == 'doi':
                paper['doi'] = identifier.text
            elif id_type == 'pmid':
                paper['pmid'] = identifier.text
            else:
                # if never found, remain at None from initialization
                pass

        # collect aulast and auinint
        aulast = []
        auinit = []
        contribs = article.findall('./front/article-meta/contrib-group/contrib')
        # if contrib is not found then loop is skipped
        for contrib in contribs:
            contrib_type = contrib.get('contrib-type')
            if contrib_type == 'author':
                surname = contrib.find('./name/surname')
                if surname is not None:
                    # then it was found
                    aulast.append(surname.text)
                else:
                    aulast.append(None)

                # multiple given names? this takes first one
                given_name = contrib.find('./name/given-names')
                if given_name is not None:
                    # then it was found
                    auinit.append(given_name.text[0])
                else:
                    auinit.append(None)
        paper['aulast'] = aulast
        paper['auinit'] = auinit

        # collect date
        pub_dates = article.findall('./front/article-meta/pub-date')
        # if pub-date is not found then loop is skipped
        for pub_date in pub_dates:
            pub_type = pub_date.get('pub-type')
            print pub_type
            if pub_type == 'collection':
                year = pub_date.find('./year')
                if year is not None:
                    # then it was found
                    paper['date'] = year.text
                else:
                    paper['date'] = None

        meta_list.append(paper)

        # construct ayjid
        paper['ayjid'] = create_ayjid(**paper)  # THIS IS BROKEN.

        # citations
        citations_list = []

        # element-citation handling different from mixed-citation handling
        citations = article.findall('./back/ref-list/ref/element-citation')
        for cite in citations:
            cite_dict = ds.Paper()

            # simple meta data
            for key in cit_meta_loc.iterkeys():
                key_data = cite.find(cit_meta_loc[key])
                if key_data is not None:
                    paper[key] = key_data.text
                else:
                    paper[key] = None

            # doi and pmid
            pub_id = cite.find('./pub-id')
            if pub_id is not None:
                pub_id_type = pub_id.get('pub-id-type')
                if pub_id_type == 'doi':
                    cite_dict['doi'] = pub_id.text
                elif pub_id_type == 'pmid':
                    cite_dict['pmid'] = pub_id.text

            # aulast and auinit
            cite_aulast = []
            cite_auinit = []

            # determine if person group is authors
            person_group = cite.find('./person-group')
            if person_group is not None:
                group_type = person_group.get('person-group-type')
            else:
                group_type = None

            # then add the authors to the cite_dict
            if group_type == 'author':
                names = person_group.findall('./name')
                for name in names:
                    # add surname
                    surname = name.find('./surname')
                    if surname is not None:
                        # then it was found
                        cite_aulast.append(surname.text)
                    else:
                        cite_aulast.append(None)

                    # add given names
                    given_names = name.find('./given-names')
                    if given_names is not None:
                        # then it was found
                        cite_auinit.append(given_names.text[0])
                    else:
                        cite_auinit.append(None)

            if not cite_aulast:
                # then empty
                cite_aulast = None
            if not cite_auinit:
                # then empty
                cite_auinit = None

            cite_dict['aulast'] = cite_aulast
            cite_dict['auinit'] = cite_auinit

            citations_list.append(cite_dict)
        # end cite loop

        paper['citations'] = citations_list

        paper['accession'] = accession

        meta_list.append(paper)
    # end article loop

    return meta_list

def _expand_pubmed(meta_list):
    """Future (not implemented).

    Given a list of first-level meta dicts and their second-level meta dicts,
    first['citations'], expand the network by adding the second-level meta
    dicts to the first level. That is, for the second-level meta dicts with
    sufficient information (either a DOI, PubMed ID, enough metadata to
    query for a DOI, etc.), query PubMed for their more expansive set
    of meta data, most notably their citation data, parse the associated xml,
    and append their :class:`.Paper` to the meta_list.

    Parameters
    ----------
    meta_list : list
        A list of :class:`.Paper` instances.

    Returns
    -------
    list : list
        A list of :class:`.Paper` instances.

    Notes
    -----
    Do something about the redundent information about them stored still in the
    first level?

    """
    pass



"""
Reader for output from topic modeling with MALLET.
"""

import csv
import numpy as np
from tethne.data import LDAModel, Paper
from tethne.utilities import Dictionary


def load(top_doc, word_top, topic_keys, Z, metadata=None, metadata_key='doi'):
    """
    Parse results from LDA modeling with MALLET.

    MALLET's LDA topic modeling algorithm produces a collection of output files.
    :func:`.read` takes the topic-document and (sparse) word-topic matrices, as
    tab-separated value files, along with a metadata file that maps
    each MALLET document id to a :class:`.Paper`\, using the `metadata_key`.

    Parameters
    ----------
    top_doc : string
        Path to topic-document datafile generated with --output-doc-topics.
    word_top : string
        Path to word-topic datafile generated with --word-topic-counts-file.
    topic_keys : string
        Path to topic-keys datafile generated with --output-topic-keys.
    Z : int
        Number of topics.
    metadata : string (optional)
        Path to tab-separated metadata file with IDs and :class:`.Paper` keys.

    Returns
    -------
    ldamodel : :class:`.LDAModel`

    """

    td = _handle_top_doc(top_doc, Z)
    wt,vocabulary = _handle_word_top(word_top, Z)
    tk = _handle_topic_keys(topic_keys)

    if metadata is not None:
        md = _handle_metadata(metadata)
    else:
        md = None

    ldamodel = LDAModel(td, wt, tk, md, vocabulary)

    return ldamodel

def read(top_doc, word_top, topic_keys, Z, metadata=None, metadata_key='doi'):
    """
    Generates :class:`.Paper` objects from Mallet output.

    Each :class:`.Paper` is assigned a topic vector.

    Parameters
    ----------
    top_doc : string
        Path to topic-document datafile generated with --output-doc-topics.
    word_top : string
        Path to word-topic datafile generated with --word-topic-counts-file.
    topic_keys : string
        Path to topic-keys datafile generated with --output-topic-keys.
    Z : int
        Number of topics.
    metadata : string (optional)
        Path to tab-separated metadata file with IDs and :class:`.Paper` keys.

    Returns
    -------
    papers : list
        List of :class:`.Paper`
    """

    ldamodel = load(top_doc, word_top, topic_keys, Z, metadata, metadata_key)
    D = ldamodel.doc_topic.shape[0]

    papers = []

    for d in xrange(D):
        p = Paper()
        p[metadata_key] = ldamodel.metadata[d]  # e.g. doi, wosid
        p['topics'] = (ldamodel.doc_topic[d,:], ldamodel.top_keys)
        papers.append(p)

    return papers

def _handle_top_doc(path, Z):
    """
    Returns
    -------
    td : Numpy array
        Rows are documents, columns are topics. Rows sum to ~1.
    """
    documents = {}

    with open(path, "rb") as f:
        reader = csv.reader(f, delimiter='\t')
        lines = [ line for line in reader ][1:] # Discard header row.
        for line in lines:
            t = line[2:]
            tops = []
            for i in xrange(0,len(t)-1,2):
                tops.append( (int(t[i]), float(t[i+1])) )
                #topics.add(int(t[i]))
            documents[int(line[0])] = (line[1], tops)

    td = np.zeros( (len(documents), Z) )

    for d, value in documents.iteritems():
        for t,p in value[1]:
            td[d,t] = p

    return td

def _handle_word_top(path, Z):
    """
    Returns
    -------
    wt : Numpy array
        Rows are topics, columns are words. Rows sum to ~1.
    vocabulary : :class:`.Dictionary`
        Maps words to word-indices in wt.
    """
    words = {}
    topics = set()

    with open(path, "rb") as f:
        reader = csv.reader(f, delimiter=' ')
        for line in reader:
            tc = { int(tuple(l.split(':'))[0]):float(tuple(l.split(':'))[1]) \
                    for l in line[2:] }
            words[int(line[0])] = ( line[1], tc )

    wt = np.zeros((Z, len(words)))

    for w, values in words.iteritems():
        for t,p in values[1].iteritems():
            wt[t,w] = p

    # Normalize
    for t in xrange(Z):
        wt[t,:] /= np.sum(wt[t,:])

    # Build vocabulary
    vocabulary = Dictionary()
    for k,v in words.iteritems():
        vocabulary[k] = v[0]

    return wt, vocabulary

def _handle_topic_keys(path):
    """
    Returns
    -------
    tk : dict
        Keys are topic indices, values are (P, terms) tuples, where terms is a
        list of strings and P is float.
    """
    tk = {}

    with open(path, "rb") as f:
        reader = csv.reader(f, delimiter='\t')
        for l in reader:
            tk[int(l[0])] = (float(l[1]), l[2].split())

    return tk

def _handle_metadata(path):
    """
    Returns
    -------
    md : dict
        Keys are document indices, values are identifiers from a :class:`.Paper`
        property.
    """
    md = {}

    with open(path, "rU") as f:
        reader = csv.reader(f, delimiter='\t')
        lines = [ l for l in reader ][1:]
        for l in lines:
            md[int(l[0])] = l[1]

    return md



import tethne.writers as wr
import tethne.utilities as util
import unittest
import networkx as nx
import os

class TestSifWriter(unittest.TestCase):
    def setUp(self):
        self.simple_graph = nx.Graph() 
        self.simple_graph.add_path(range(5))

        self.multi_graph = nx.MultiGraph()
        self.multi_graph.add_path(range(5))
        self.multi_graph.add_path(range(5))

        # can a path be a person?
        self.attribs = {'brown_eyes':2,
                        'loves_travel':1,
                        'brown_hair':'lots'}
        # wat
        self.simple_attribs = nx.Graph()
        self.simple_attribs.add_path(range(5), **self.attribs)

        self.multi_attribs = nx.MultiGraph()
        self.multi_attribs.add_path(range(5), **self.attribs)
        self.multi_attribs.add_path(range(5), **self.attribs)

    def test_simple_graph(self):
        filepath = os.path.join(os.path.dirname(__file__), 'testout',
                                'simple_graph')
        wr.graph.to_sif(self.simple_graph, filepath)
        expected = ('0 rel 1\n' +
                    '1 rel 2\n' +
                    '2 rel 3\n' +
                    '3 rel 4\n')
        obtained = open(filepath + '.sif', 'r').read()
        self.assertEqual(expected, obtained)
        os.remove(filepath + '.sif')

    def test_multi_graph(self):
        filepath = os.path.join(os.path.dirname(__file__), 'testout',
                                'multi_graph')
        wr.graph.to_sif(self.multi_graph, filepath)
        expected = ('0 0 1\n' +
                    '0 1 1\n' +
                    '1 0 2\n' +
                    '1 1 2\n' +
                    '2 0 3\n' +
                    '2 1 3\n' +
                    '3 0 4\n' +
                    '3 1 4\n')
        obtained = open(filepath + '.sif', 'r').read()
        self.assertEqual(expected, obtained)
        os.remove(filepath + '.sif')

    def test_simple_attribs(self):
        filepath = os.path.join(os.path.dirname(__file__), 'testout',
                                'simple_attribs')
        wr.graph.to_sif(self.simple_attribs, filepath)

        # test sif
        expected = ('0 rel 1\n' +
                    '1 rel 2\n' +
                    '2 rel 3\n' +
                    '3 rel 4\n')
        obtained = open(filepath + '.sif', 'r').read()
        self.assertEqual(expected, obtained)
        os.remove(filepath + '.sif')

        # test eda
        for key, value in self.attribs.iteritems():
            expected = (key + '\n' +
                        '0 (rel) 1 = ' + str(value) + '\n' +
                        '1 (rel) 2 = ' + str(value) + '\n' +
                        '2 (rel) 3 = ' + str(value) + '\n' +
                        '3 (rel) 4 = ' + str(value) + '\n')
            obtained = open(filepath + '_' + key + '.eda', 'r').read()
            self.assertEqual(expected, obtained)
            os.remove(filepath + '_' + key + '.eda')

    def test_multi_attribs(self):
        filepath = os.path.join(os.path.dirname(__file__), 'testout',
                                'multi_attribs')
        wr.graph.to_sif(self.multi_attribs, filepath)

        # test sif
        obtained = open(filepath + '.sif', 'r').read()
        expected = ('0 0 1\n' +
                    '0 1 1\n' +
                    '1 0 2\n' +
                    '1 1 2\n' +
                    '2 0 3\n' +
                    '2 1 3\n' +
                    '3 0 4\n' +
                    '3 1 4\n')
        self.assertEqual(expected, obtained)
        os.remove(filepath + '.sif')

        # test eda
        for key, value in self.attribs.iteritems():
            expected = (key + '\n' +
                        '0 (0) 1 = ' + str(value) + '\n' +
                        '0 (1) 1 = ' + str(value) + '\n' +
                        '1 (0) 2 = ' + str(value) + '\n' +
                        '1 (1) 2 = ' + str(value) + '\n' +
                        '2 (0) 3 = ' + str(value) + '\n' +
                        '2 (1) 3 = ' + str(value) + '\n' +
                        '3 (0) 4 = ' + str(value) + '\n' +
                        '3 (1) 4 = ' + str(value) + '\n')
            obtained = open(filepath + '_' + key + '.eda', 'r').read()
            self.assertEqual(expected, obtained)
            os.remove(filepath + '_' + key + '.eda')

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



"""
Tethne is a package for analyzing citation data from the Web of Science.
Modules within Tethne can generate a variety of networks, such as
bibliographic coupling, citation, author-paper, and co-author networks,
using networkx.

.. autosummary::

   tethne.analyze
   tethne.builders
   tethne.data
   tethne.matrices
   tethne.networks
   tethne.readers
   tethne.utilities
   tethne.writers
   
"""



"""
The :mod:`tethne.analyze` sub-package provides additional analysis methods not
provided by NetworkX, as well as methods for using NetworkX algorithms on
an entire :class:`.GraphCollection`\.

.. autosummary::

   collection
   graph

"""

import tethne.analyze.collection
import tethne.analyze.graph



"""
Write NetworkX graphs to structured and unstructured network file formats.

Many methods simply invoke equivalent methods in NetworkX.

.. autosummary::

   to_gexf
   to_graphml
   to_sif

"""

import networkx as nx
import csv

def to_sif(graph, output_path):
    """
    Generates Simple Interaction Format output file from provided graph.

    The SIF specification is described 
    `here <http://wiki.cytoscape.org/Cytoscape_User_Manual/Network_Formats>`_.

    :func:`.to_sif` will generate a .sif file describing the network, and a few
    .eda and .noa files containing edge and node attributes, respectively. These
    are equivalent to tab-delimited tables, and can be imported as such in
    Cytoscape 3.0.
    
    Parameters
    ----------
    graph : networkx.Graph
        The Graph to be exported to SIF.
    output_path : str
        Full path, including filename (without suffix).
        e.g. using "./graphFolder/graphFile" will result in a SIF file at
        ./graphFolder/graphFile.sif, and corresponding .eda and .noa files.

    """
    
    graph = _strip_list_attributes(graph)
    
    if output_path[-4:] == ".sif":
        output_path = output_path[:-4]

    if nx.number_of_nodes(graph) == 0:
        # write an empty file (for a non-existant graph)
        f = open(output_path + ".sif","w")
        f.write("")
        f.close()

    else:
        # write node files
        nodes = graph.nodes(data=True)
        for node in nodes:
            node_name = node[0]
            node_attribs = node[1]
            for key, value in node_attribs.iteritems():
                # generate a node attribute file for each node attribute
                if node == nodes[0]:
                    # first node, overwrite file
                    with open(output_path + "_" + key + ".noa",
                              "w") as f:
                        f.write(str(key) + '\n')
                        f.write(str(node_name).replace(" ","_") + " = " +
                                str(value) + "\n")
                else:
                    # not first, append file
                    with open(output_path + "_" + key + ".noa",
                              "a") as f:
                        f.write(str(node_name).replace(" ","_") + " = " +
                                str(value) + "\n")

        if nx.number_of_edges(graph) == 0:
            # write an empty graph to a .sif file (just its nodes)
            for node in nodes:
                node_name = node[0]
                if node == nodes[0]:
                    # first node, overwrite file
                    with open(output_path + ".sif","w") as f:
                        f.write(str(node_name).replace(" ","_") + "\n")
                else:
                    # not first, append file
                    with open(output_path + ".sif","a") as f:
                        f.write(str(node_name).replace(" ","_") + "\n")

        else:
            # write the graph to a .sif file as well as other edge
            # attribute files

            if graph.is_multigraph():
                # then the NetworkX graph supports multiple interaction
                # types just like the .sif format
                edges = graph.edges(data=True, keys=True)
                edge_attribs = set()
                for edge in edges:
                    for key in edge[3].iterkeys():
                        edge_attribs.add(key)

                # create edge attribute files
                for attrib in edge_attribs:
                    str_attrib = str(attrib)
                    with open(output_path + '_' + str_attrib + ".eda","w") as f:
                        f.write(str(attrib) + "\n")

                # add data to eda files and write sif file
                with open(output_path + '.sif', 'w') as f:
                    for edge in edges:
                        node1 = str(edge[0]).replace(" ", "_")
                        node2 = str(edge[1]).replace(" ", "_")
                        intr_type = str(edge[2]).replace(" ", "_")
                        sif_line = node1 + ' ' + intr_type + ' ' + node2 + '\n'
                        f.write(sif_line)

                        for attrib, value in edge[3].iteritems():
                            eda_line = (node1 + ' (' + intr_type + ') ' +
                                        node2 + ' = ' + str(value) + '\n')
                            with open(output_path + '_' + str(attrib) + '.eda',
                                      'a') as g:
                                g.write(eda_line)

            else:
                # then we support only one interaction type 'rel'
                edges = graph.edges(data=True)
                edge_attribs = set()
                for edge in edges:
                    for key in edge[2].iterkeys():
                        edge_attribs.add(key)

                # create edge attribute files
                for attrib in edge_attribs:
                    str_attrib = str(attrib)
                    with open(output_path + '_' + str_attrib + ".eda","w") as f:
                        f.write(str(attrib) + "\n")

                # add data to eda files and write sif file
                with open(output_path + '.sif', 'w') as f:
                    for edge in edges:
                        node1 = str(edge[0]).replace(" ", "_")
                        node2 = str(edge[1]).replace(" ", "_")
                        intr_type = 'rel'
                        sif_line = node1 + ' ' + intr_type + ' ' + node2 + '\n'
                        f.write(sif_line)

                        for attrib, value in edge[2].iteritems():
                            eda_line = (node1 + ' (' + intr_type + ') ' +
                                        node2 + ' = ' + str(value) + '\n')
                            with open(output_path + '_' + str(attrib) +
                                      '.eda', 'a') as g:
                                g.write(eda_line)

def to_gexf(graph, output_path):
    """Writes graph to `GEXF <http://gexf.net>`_.
    
    Uses the NetworkX method
    `write_gexf <http://networkx.lanl.gov/reference/generated/networkx.readwrite.gexf.write_gexf.html>`_.
    
    Parameters
    ----------
    graph : networkx.Graph
        The Graph to be exported to GEXF.
    output_path : str
        Full path, including filename (without suffix).
        e.g. using "./graphFolder/graphFile" will result in a GEXF file at
        ./graphFolder/graphFile.gexf.
    """
    graph = _strip_list_attributes(graph)
    
    nx.write_gexf(graph, output_path + ".gexf")


def to_graphml(graph, output_path):
    """Writes graph to `GraphML <http://graphml.graphdrawing.org/>`_.
    
    Uses the NetworkX method 
    `write_graphml <http://networkx.lanl.gov/reference/generated/networkx.readwrite.graphml.write_graphml.html>`_.

    Parameters
    ----------
    graph : networkx.Graph
        The Graph to be exported to GraphML.
    output_path : str
        Full path, including filename (without suffix).
        e.g. using "./graphFolder/graphFile" will result in a GraphML file at
        ./graphFolder/graphFile.graphml.
    """
    graph = _strip_list_attributes(graph)
    
    nx.write_graphml(graph, output_path + ".graphml")

#def to_csv(file, delim=","):
#    '''
#    Parameters
#    ----------
#    file : string
#        Path to output file (will be created).
#    delim : string
#        String to use as field delimiter (default is ',').
#
#    Notes
#    -----
#    TODO: should operate on a (provided) graph. Still uses old library approach.
#    '''
#    
#    graph = _strip_list_attributes(graph)
#    
#    f = open(file, "w")
#
#    # Headers
#    f.write("Identifier" + delim + "Title" + delim + "Authors" +
#            delim + "WoS Identifier" + delim + "Journal" + delim +
#            "Volume" + delim + "Page" + delim + "DOI" + delim +
#            "Num Authors\n")
#    for entry in self.library:
#        # Authors are separated by a colon -> : <-
#        authors = ""
#        for author in entry.meta['AU']:
#            authors += ":" + author
#        authors = authors[1:]
#        datum = (entry.identifier + delim + entry.meta['TI'][0] +
#                 delim + authors + delim + entry.wosid + delim +
#                 entry.meta['SO'][0])
#        if 'VL' in entry.meta:
#            datum += delim + entry.meta['VL'][0]
#            if 'BP' in entry.meta:
#                datum += delim + entry.meta['BP'][0]
#            else:
#                datum += delim
#        else:
#            datum += delim + delim
#        if 'DI' in entry.meta:
#            datum += delim + entry.meta['DI'][0]
#        else:
#            datum += delim
#        datum += delim + str(entry.meta['num_authors'])
#        f.write(datum + "\n")
#    f.close()

def to_table(graph, path):

    graph = _strip_list_attributes(graph)

    # Edge list.
    with open(path + "_edges.csv", "wb") as f:
        edges = graph.edges(data=True)
        writer = csv.writer(f, delimiter='\t')
        
        # Header.
        writer.writerow(['source','target'] + [ k for k in edges[0][2].keys() ])
        
        # Values.
        for e in edges:
            writer.writerow([ e[0], e[1]] + [ v for v in e[2].values() ] )
            
    # Node attributes.
    with open(path + "_nodes.csv", "wb") as f:
        nodes = graph.nodes(data=True)
        writer = csv.writer(f, delimiter='\t')
        
        # Header.
        writer.writerow(['node'] + [ k for k in nodes[0][1].keys() ])
        
        # Values.
        for n in nodes:
            writer.writerow([ n[0] ] + [ v for v in n[1].values() ])


def _strip_list_attributes(G):
    for n in G.nodes(data=True):
        for k,v in n[1].iteritems():
            if type(v) is list:
                G.node[n[0]][k] = str(v)
    for e in G.edges(data=True):
        for k,v in e[2].iteritems():
            if type(v) is list:
                G.edge[e[0]][e[1]][k] = str(v)

    return G





"""
Helper functions for generating networks.

.. autosummary::

   citation_count
   simplify_multigraph
   top_cited
   top_parents

"""

import networkx as nx
import operator
import numpy as np
#import tethne.utilities as util  # Pylint Warnings
#import tethne.data as ds

def simplify_multigraph(multigraph, time=False):
    """
    Simplifies a graph by condensing multiple edges between the same node pair
    into a single edge, with a weight attribute equal to the number of edges.

    Parameters
    ----------
    graph : networkx.MultiGraph
        E.g. a coauthorship graph.
    time : bool
        If True, will generate 'start' and 'end' attributes for each edge,
        corresponding to the earliest and latest 'date' values for that edge.

    Returns
    -------
    graph : networkx.Graph
        A NetworkX :class:`.graph` .

    """

    graph = nx.Graph()

    for node in multigraph.nodes(data=True):
        u = node[0]
        node_attribs = node[1]
        graph.add_node(u, node_attribs)


        for v in multigraph[u]:
            edges = multigraph.get_edge_data(u, v) # Dict.

            edge_attribs = { 'weight': len(edges) }

            if time:    # Look for a date in each edge.
                start = 3000
                end = 0
                found_date = False
                for edge in edges.values():
                    try:
                        found_date = True
                        if edge['date'] < start:
                            start = edge['date']
                        if edge['date'] > end:
                            end = edge['date']
                    except KeyError:    # No date to be found.
                        pass

                if found_date:  # If no date found, don't add start/end atts.
                    edge_attribs['start'] = start
                    edge_attribs['end'] = end

            graph.add_edge(u, v, edge_attribs)

    return graph

def citation_count(papers, key='ayjid', verbose=False):
    """
    Generates citation counts for all of the papers cited by papers.

    Parameters
    ----------
    papers : list
        A list of :class:`.Paper` instances.
    key : str
        Property to use as node key. Default is 'ayjid' (recommended).
    verbose : bool
        If True, prints status messages.

    Returns
    -------
    counts : dict
        Citation counts for all papers cited by papers.
    """

    if verbose:
        print "Generating citation counts for "+str(len(papers))+" papers..."

    counts = {}
    for P in papers:
        if P['citations'] is not None:
            for p in P['citations']:
                try:
                    counts[p[key]] += 1
                except KeyError:
                    counts[p[key]] = 1

    return counts

def top_cited(papers, topn=20, verbose=False):
    """
    Generates a list of the topn (or topn%) most cited papers.

    Parameters
    ----------
    papers : list
        A list of :class:`.Paper` instances.
    topn : int or float {0.-1.}
        Number (int) or percentage (float) of top-cited papers to return.
    verbose : bool
        If True, prints status messages.

    Returns
    -------
    top : list
        A list of 'ayjid' keys for the topn most cited papers.
    counts : dict
        Citation counts for all papers cited by papers.
    """

    if verbose:
        print "Finding top "+str(topn)+" most cited papers..."

    counts = citation_count(papers, verbose=verbose)

    if type(topn) is int:
        n = topn
    elif type(topn) is float:
        n = int(np.around(topn*len(counts), decimals=-1))
    top = dict(sorted(counts.iteritems(),
                       key=operator.itemgetter(1),
                       reverse=True)[:n]).keys()

    return top, counts

def top_parents(papers, topn=20, verbose=False):
    """
    Returns a list of :class:`.Paper` that cite the topn most cited papers.

    Parameters
    ----------
    papers : list
        A list of :class:`.Paper` objects.
    topn : int or float {0.-1.}
        Number (int) or percentage (float) of top-cited papers.
    verbose : bool
        If True, prints status messages.

    Returns
    -------
    papers : list
        A list of :class:`.Paper` objects.
    top : list
        A list of 'ayjid' keys for the topn most cited papers.
    counts : dict
        Citation counts for all papers cited by papers.
    """

    if verbose:
        print "Getting parents of top "+str(topn)+" most cited papers."

    top, counts = top_cited(papers, topn=topn, verbose=verbose)
    papers = [ P for P in papers if P['citations'] is not None ]
    parents = [ P for P in papers if len(
                    set([ c['ayjid'] for c in P['citations'] ]) &
                    set(top) ) > 0 ]

    if verbose:
        print "Found " + str(len(parents)) + " parents."

    return parents, top, counts


"""
Classes for handling bibliographic data.

.. autosummary::

   Paper
   DataCollection
   GraphCollection
   LDAModel
   
"""

import networkx as nx
import pickle as pk
from cStringIO import StringIO
from pprint import pprint
import sys
import numpy as np


class Paper(object):
    """
    Base class for Papers. 
    
    Behaves just like a dict, but enforces a limited vocabulary of keys, and 
    specific data types.

    The following fields (and corresponding data types) are allowed:
    
    ===========     =====   ====================================================
    Field           Type    Description
    ===========     =====   ====================================================
    aulast          list    Authors' last name, as a list.
    auinit          list    Authors' first initial as a list.
    institution     dict    Institutions with which the authors are affiliated.
    atitle          str     Article title.
    jtitle          str     Journal title or abbreviated title.
    volume          str     Journal volume number.
    issue           str     Journal issue number.
    spage           str     Starting page of article in journal.
    epage           str     Ending page of article in journal.
    date            int     Article date of publication.
    country         dict    Author-Country mapping.
    citations       list    A list of :class:`.Paper` instances.
    ayjid           str     First author's name (last fi), pubdate, and journal.
    doi             str     Digital Object Identifier.
    pmid            str     PubMed ID.
    wosid           str     Web of Science UT fieldtag value.
    accession       str     Identifier for data conversion accession.
    ===========     =====   ====================================================

    None values are also allowed for all fields.
    """

    def __init__(self):
        """
        Defines keys, and acceptable data types for values.
        """
        self.internal = {
                            'aulast':None,
                            'auinit':None,
                            'institutions':None,
                            'atitle':None,
                            'jtitle':None,
                            'volume':None,
                            'issue':None,
                            'spage':None,
                            'epage':None,
                            'date':None,
                            'citations':None,
                            'country':None,
                            'ayjid':None,
                            'doi':None,
                            'pmid':None,
                            'wosid':None,
                            'abstract':None,
                            'accession':None,
                            'topics':None    }

        self.list_fields = [ 'aulast',
                             'auinit',
                             'citations' ]

        self.string_fields = [ 'atitle',
                               'jtitle',
                               'volume',
                               'issue',
                               'spage',
                               'epage',
                               'ayjid',
                               'doi',
                               'pmid',
                               'wosid',
                               'abstract',
                               'accession' ]

        self.int_fields = [ 'date' ]

        self.dict_fields = [ 'institutions' ]

    def __setitem__(self, key, value):
        """
        Enforces limited vocabulary of keys, and acceptable data types for
        values.
        """

        vt = type(value)
        ks = str(key)

        if key not in self.internal.keys():
            raise KeyError(ks + " is not a valid key in Paper.")
        elif key in self.list_fields and vt is not list and value is not None:
            raise ValueError("Value for field '"+ ks +"' must be a list.")
        elif key in self.string_fields and vt is not str and value is not None:
            raise ValueError("Value for field '"+ ks +"' must be a string.")
        elif key in self.int_fields and vt is not int and value is not None:
            raise ValueError("Value for field '"+ ks +"' must be an integer.")
        elif key in self.dict_fields and vt is not dict and value is not None:
            raise ValueError("Value for field '"+ ks +"' must be a dictionary.")
        else:
            self.internal[key] = value

    def __getitem__(self, key):
        return self.internal[key]

    def __delitem__(self, key):
        del self.internal[key]

    def __len__(self):
        return len(self.internal)

    def keys(self):
        """Returns the keys of the :class:`.Paper`'s metadata fields."""
        return self.internal.keys()

    def values(self):
        """Returns the values of the :class:`.Paper`'s metadata fields."""
        return self.internal.values()

    def iteritems(self):
        """Returns an iterator for the :class:`.Paper`'s metadata fields"""
        return self.internal.iteritems()
    
    def authors(self):
        """Returns a list of author names (FI LAST)."""
        
        auths = []
        for i in xrange(len(self.internal['aulast'])):
            au = self.internal['auinit'][i] + ' ' +  self.internal['aulast'][i]
            auths.append( au.upper() )
        return auths

class DataCollection(object):
    """
    A :class:`.DataCollection` organizes :class:`.Paper`\s for analysis.
    
    The :class:`.DataCollection` is initialized with some data, which is indexed
    by a key in :class:`.Paper` (default is wosid). The :class:`.DataCollection`
    can then be sliced ( :func:`DataCollection.slice` ) by other keys in
    :class:`.Paper` .
    
    **Usage**
    
    .. code-block:: python

       >>> import tethne.readers as rd
       >>> data = rd.wos.read("/Path/to/wos/data.txt")
       >>> data += rd.wos.read("/Path/to/wos/data2.txt")    # Two accessions.
       >>> from tethne.data import DataCollection
       >>> D = DataCollection(data) # Indexed by wosid, by default.
       >>> D.slice('date', 'time_window', window_size=4)
       >>> D.slice('accession')
       >>> D
       <tethne.data.DataCollection at 0x10af0ef50>
        
    """
    
    def __init__(self, data, index_by='wosid'):
        self.axes = {}
        self.index_by = index_by
        self.datakeys = data[0].keys()
        
        if type(data[0]) is not Paper:
            raise(ValueError("Data must contain tethne.data.Paper objects."))

        if index_by not in self.datakeys:
            raise(KeyError(str(index_by) + " not a valid key in data."))
        
        self.data = { p[index_by]:p for p in data }
    
    def slice(self, key, method=None, **kwargs):
        """
        Slices data by key, using method (if applicable).
        
        Parameters
        ----------
        key : str
            key in :class:`.Paper` by which to slice data.
        method : str (optional)
            Dictates how data should be sliced. See table for available methods.
            If key is 'date', default method is time_period with window_size and
            step_size of 1.
        kwargs : kwargs
            See methods table, below.
            
        Notes
        -----

        Methods available for slicing DataCollections:

        ===========    =============================    =======    =============
        Method         Description                      Key        kwargs
        ===========    =============================    =======    =============
        time_window    Slices data using a sliding      date       window_size
                       time-window. Dataslices are                 step_size
                       indexed by the start of the 
                       time-window.
        time_period    Slices data into time periods    date       window_size
                       of equal length. Dataslices
                       are indexed by the start of
                       the time period.
        ===========    =============================    =======    =============

        Avilable kwargs:

        ===========    ======   ================================================
        Argument       Type     Description
        ===========    ======   ================================================
        window_size    int      Size of time-window or period, in years
                                (default = 1).
        step_size      int      Amount to advance time-window or period in each
                                step (ignored for time_period).
        cumulative     bool     If True, the data from each successive slice 
                                includes the data from all preceding slices.
                                Only applies if key is 'date' (default = False).
        ===========    ======   ================================================            
        """
        
        if key == 'date':
            if method == 'time_window':
                kw = {  'window_size': kwargs.get('window_size', 1),
                        'step_size': 1 }
                self.axes[key] = self._time_slice(**kw)

            elif method == 'time_period' or method is None:
                kw = {  'window_size': kwargs.get('window_size', 1),
                        'step_size': kwargs.get('window_size', 1),
                        'cumulative': kwargs.get('cumulative', False) }

                self.axes[key] = self._time_slice(**kw)
            else:
                raise(ValueError(str(method) + " not a valid slicing method."))
            
        elif key == 'author':
            self.axes[key] = {}
            for i,p in self.data.iteritems():
                for a in p.authors():
                    try:
                        self.axes[key][a].append(i)
                    except KeyError:
                        self.axes[key][a] = [i]
        elif key in self.datakeys: # e.g. 'jtitle'
            self.axes[key] = {}
            for i,p in self.data.iteritems():
                try:
                    self.axes[key][p[key]].append(i)
                except KeyError:
                    self.axes[key][p[key]] = [i]
        else:
            raise(KeyError(str(key) + " not a valid key in data."))
        
    def _time_slice(self, **kwargs):
        """
        Slices data by date.

        If step_size = 1, this is a sliding time-window. If step_size =
        window_size, this is a time period slice.
        
        Parameters
        ----------
        kwargs : kwargs
            See table, below.
            
        Returns
        -------
        slices : dict
            Keys are start date of time slice, values are :class:`.Paper`
            indices (controlled by index_by argument in 
            :func:`.DataCollection.__init__` )
        
        Notes
        -----
        
        Avilable kwargs:

        ===========    ======   ================================================
        Argument       Type     Description
        ===========    ======   ================================================
        window_size    int      Size of time-window or period, in years
                                (default = 1).
        step_size      int      Amount to advance time-window or period in each
                                step (ignored for time_period).
        cumulative     bool     If True, the data from each successive slice 
                                includes the data from all preceding slices.
                                Only applies if key is 'date' (default = False).                                
        ===========    ======   ================================================           
        
        """

        window_size = kwargs.get('window_size', 1)
        step_size = kwargs.get('step_size', 1)
        start = kwargs.get('start', min([ p['date'] 
                                                 for p in self.data.values() ]))
        end = kwargs.get('start', max([ p['date'] 
                                                 for p in self.data.values() ]))
        cumulative = kwargs.get('cumulative', False)

        slices = {}
        for i in xrange(start, end-window_size+2, step_size):
            slices[i] = [ k for k,p in self.data.iteritems() 
                           if i <= p['date'] < i + window_size ]
            if cumulative:
                try:
                    slices[i] += slices[i-1]
                except KeyError:
                    pass

        return slices
        
    def indices(self):
        """
        Yields a list of indices of all papers in this :class:`.DataCollection`
        
        Returns
        -------
        list
            List of indices.
        """
        
        return self.data.keys()
    
    def papers(self):
        """
        Yield the complete set of :class:`.Paper` instances in this
        :class:`.DataCollection` .
        
        Returns
        -------
        papers : list
            A list of :class:`.Paper`
        """
        
        return self.data.values()
    
    def get_slices(self, key, papers=False):
        """
        Yields slices for key.
        
        Parameters
        ----------
        key : str
            Key from :class:`.Paper` that has previously been used to slice data
            in this :class:`.DataCollection` .
        
        Returns
        -------
        slices : dict
            Keys are slice indices. If papers is True, values are lists of
            :class:`.Paper` instances; otherwise returns paper indices (e.g.
            'wosid').

        Raises
        ------
        RuntimeError : DataCollection has not been sliced.
        KeyError : Data has not been sliced by [key] 
                    
        """

        if len(self.axes) == 0:
            raise(RuntimeError("DataCollection has not been sliced."))        
        if key not in self.axes.keys():
            raise(KeyError("Data has not been sliced by " + str(key)))
        
        slices = self.axes[key]
         
        if papers:  # Retrieve Papers.
            for k,v in slices.iteritems():
                slices[k] = [ self.data[i] for i in v ]
        
        return slices

    def get_slice(self, key, index, papers=False):
        """
        Yields a specific slice.
        
        Parameters
        ----------
        key : str
            Key from :class:`.Paper` that has previously been used to slice data
            in this :class:`.DataCollection` .
        index : str or int
            Slice index for key (e.g. 1999 for 'date').
        
        Returns
        -------
        slice : list
            List of paper indices in this :class:`.DataCollection` , or (if
            papers is True) a list of :class:`.Paper` instances.

        Raises
        ------
        RuntimeError : DataCollection has not been sliced.
        KeyError : Data has not been sliced by [key] 
        KeyError : [index] not a valid index for [key]
        
        """

        if len(self.axes) == 0:
            raise(RuntimeError("DataCollection has not been sliced."))        
        if key not in self.axes.keys():
            raise(KeyError("Data has not been sliced by " + str(key)))
        if index not in self.axes[key].keys():
            raise(KeyError(str(index) + " not a valid index for " + str(key)))
        
        slice = self.axes[key][index]
        
        if papers:
            return [ self.data[s] for s in slice ]
        return slice
    
    def get_by(self, key_indices, papers=False):
        """
        Given a set of (key, index) tuples, return the corresponding subset of
        :class:`.Paper` indices (or :class:`.Paper` instances themselves, if 
        papers is True).
        
        Parameters
        ----------
        key_indices : list
            A list of (key, index) tuples.
        
        Returns
        -------
        plist : list
            A list of paper indices, or :class:`.Paper` instances.
        
        Raises
        ------
        RuntimeError : DataCollection has not been sliced.

        """

        if len(self.axes) == 0:
            raise(RuntimeError("DataCollection has not been sliced."))
        
        slices = []
        for k,i in key_indices:
            slice = set(self.get_slice(k,i))
            slices.append(slice)

        plist = list( set.intersection(*slices) )
        
        if papers:
            return [ self.data[s] for s in plist ]
        return plist
    
    def _get_slice_i(self, key, i):
        return self.axes[key].values()[i]
        
    def _get_by_i(self, key_indices):
        slices = []
        for k, i in key_indices:
            slice = set(self._get_slice_i(k, i))
            slices.append(slice)
        
        return list( set.intersection(*slices) )
    
    def get_axes(self):
        """
        Returns a list of all slice axes for this :class:`.DataCollection` .
        """
        
        return self.axes.keys()
    
    def N_axes(self):
        """
        Returns the number of slice axes for this :class:`.DataCollection` .
        """
        
        return len(self.axes.keys())
    
    def distribution(self):
        """
        Returns a Numpy array describing the number of :class:`.Paper`
        associated with each slice-coordinate.
        
        WARNING: expensive for a :class:`.DataCollection` with many axes or
        long axes. Consider using :func:`.distribution_2d` .
        
        Returns
        -------
        dist : Numpy array
            An N-dimensional array. Axes are given by 
            :func:`DataCollection.get_axes` and values are the number of
            :class:`.Paper` at that slice-coordinate.
            
        Raises
        ------
        RuntimeError : DataCollection has not been sliced.
        """
        
        if len(self.axes) == 0:
            raise(RuntimeError("DataCollection has not been sliced."))
        
        shape = tuple( len(v) for v in self.axes.values() )
        dist = np.zeros(shape)
        axes = self.get_axes()

        for indices in np.ndindex(shape):
            dist[indices] = len( self._get_by_i(zip(axes, indices)))
            
        return dist
            
    def distribution_2d(self, x_axis, y_axis):
        """
        Returns a Numpy array describing the number of :class:`.Paper`
        associated with each slice-coordinate, for x and y axes spcified.

        Returns
        -------
        dist : Numpy array
            A 2-dimensional array. Values are the number of
            :class:`.Paper` at that slice-coordinate.
            
        Raises
        ------
        RuntimeError : DataCollection has not been sliced.
        KeyError: Invalid slice axes for this DataCollection.
        """
        
        if len(self.axes) == 0:
            raise(RuntimeError("DataCollection has not been sliced."))
        if x_axis not in self.get_axes() or y_axis not in self.get_axes():
            raise(KeyError("Invalid slice axes for this DataCollection."))
        
        x_size = len(self.axes[x_axis])
        y_size = len(self.axes[y_axis])
        shape = (x_size, y_size)
        dist = np.zeros(shape)
        
        for i in xrange(x_size):
            for j in xrange(y_size):
                dist[i, j] = len(self._get_by_i([(x_axis, i),(y_axis, j)]))
        
        return dist

class GraphCollection(object):
    """
    Collection of NetworkX :class:`nx.classes.graph.Graph` objects, 
    organized by some index (e.g. time). 
    
    A :class:`.GraphCollection` can be generated using classes in the
    :mod:`tethne.builders` module. See 
    :ref:`generate-graphcollection` for details.
    
    """

    def __init__(self):
        self.graphs = {}
        self.metadata = {}
        self.edge_list = []
        self.node_list = []

        return

    def __setitem__(self, index, graph, metadata=None):
        """
        The value param can be either a Graph, or a (Graph, metadata) tuple.
        Metadata can be anything, but is probably most profitably a dictionary.

        Parameters
        ----------
        index
            This can be anything used to refer to the graph.
        graph : :class:`.nx.classes.graph.Graph`

        Raises
        ------
        ValueError : Graph must be of type networkx.classes.graph.Graph
            If value is not a Graph.
        """

        if type(graph) is not nx.classes.graph.Graph:
            raise(ValueError("Graph must be type networkx.classes.graph.Graph"))

        self.graphs[index] = graph
        self.metadata[index] = metadata

    def __getitem__(self, key):
        return self.graphs[key]

    def __delitem__(self, key):
        del self.graphs[key]

    def __len__(self):
        return len(self.graphs)

    def nodes(self, overwrite=False):
        """
        Return complete set of nodes for this :class:`.GraphCollection` . 

        If this method has been called previously for this
        :class:`.GraphCollection` then will not recompute unless overwrite =
        True.

        Parameters
        ----------
        overwrite : bool
            If True, will generate new node list, even if one already exists.

        Returns
        -------
        nodes : list
            List (complete set) of node identifiers for this
            :class:`.GraphCollection` .
        """

        if len(self.node_list) == 0 or overwrite:
            nodes = set([])
            for G in self.graphs.values():
                nodes = nodes | set(G.nodes())
            self.node_list = list(nodes)
        return self.node_list

    def edges(self, overwrite=False):   # [#61512528]
        """
        Return complete set of edges for this :class:`.GraphCollection` . 

        If this method has been called previously for this
        :class:`.GraphCollection` then will not recompute unless overwrite =
        True.

        Parameters
        ----------
        overwrite : bool
            If True, will generate new node list, even if one already exists.

        Returns
        -------
        edges : list
            List (complete set) of edges for this :class:`.GraphCollection` .
        """

        if len(self.edge_list) == 0 or overwrite :
            edges = set([])
            for G in self.graphs.values():
                edges = edges | set(G.edges())
            self.edge_list = list(edges)
        return self.edge_list

    def save(self,filepath):   #[61512528]
        """
        Pickles (serializes) the :class:`.GraphCollection` .
        
        Parameters
        ----------
        filepath :
            Full path of output file.

        Raises
        -------
        PicklingError : Raised when unpicklable objects are Pickled.
        IOError : File does not exist, or cannot be opened.
        """


        # Try block if the filename is present or not.
        try:
            with open(filepath,'wb') as output:
                try:
                    pk.dump(self, output)
                except PicklingError:     # Handle the Prickling error.
                    raise PicklingError \
                            ("Pickling error: The object cannot be pickled")
        except IOError: # File does not exist, or couldn't be read.
            raise IOError("File does not exist, or cannot be opened.")


    def load(self, filepath):    #[61512528]
        """
        Loads a pickled (serialized) :class:`.GraphCollection` from filepath.

        Parameters
        ----------
        filepath : string
            Full path to pickled :class:`.GraphCollection` .

        Raises
        -------
        UnpicklingError : Raised when there is some issue in unpickling.
        IOError : File does not exist, or cannot be read.
        """

         # Handle NameError File not found.
        try:
            with open(filepath,'rb') as input: #reading in binary mode
                try:
                     obj_read = pk.load(input)
                except UnpicklingError:  # Handle unprickling error.
                    raise UnpicklingError \
                        ("UnPickling error: The object cannot be found")


        except IOError: # File does not exist, or couldn't be read.
            raise IOError("File does not exist, or cannot be read.")
        
        # Preserving the object with unpickled data
        if(obj_read):
            self.__dict__ = obj_read.__dict__

        return obj_read
        
    def compose(self):
        """
        Returns the simple union of all :class:`.Graph` in the 
        :class:`.GraphCollection` .
        
        Returns
        -------
        composed : :class:`.Graph`
            Simple union of all :class:`.Graph` in the 
            :class:`.GraphCollection` .
            
        Notes
        -----
        
        Node or edge attributes that vary over slices should be ignored.
        
        """
        
        composed = nx.Graph()
        for k, G in self.graphs.iteritems():
            composed = nx.compose(composed, G)
        
        return composed

class LDAModel(object):
    """
    Organizes parsed output from MALLET's LDA modeling algorithm.
    
    Used by :mod:`.readers.mallet`\.
    """
    
    def __init__(self, doc_topic, top_word, top_keys, metadata, vocabulary):
        """
        Initialize the :class:`.LDAModel`\.
        
        Parameters
        ----------
        doc_top : Numpy matrix
            Rows are documents, columns are topics. Values indicate the 
            contribution of a topic to a document, such that all rows sum to 
            1.0.
        top_word : Numpy matrix
            Rows are topics, columns are words. Values indicate the normalized
            contribution of each word to a topic, such that rows sum to 1.0.
        top_keys : dict
            Maps matrix indices for topics onto the top words in that topic.
        metadata : dict
            Maps matrix indices for documents onto a :class:`.Paper` key.
        """
        
        self.doc_topic = doc_topic
        self.top_word = top_word
        self.top_keys = top_keys
        self.metadata = metadata
        self.vocabulary = vocabulary
        
        self.lookup = { v:k for k,v in metadata.iteritems() }
        
    def topics_in_doc(self, d, topZ=None):
        """
        Returns a list of the topZ most prominent topics in a document.
        
        Parameters
        ----------
        d : str or int
            An identifier from a :class:`.Paper` key.
        topZ : int or float
            Number of prominent topics to return (int), or threshold (float).
            
        Returns
        -------
        topics : list
            List of (topic, proportion) tuples.
        """
        
        index = self.lookup[d]
        td = self.doc_topic[index, :]
        
        if topZ is None:
            topZ = td.shape[0]
            
        if type(topZ) is int:   # Return a set number of topics.
            top_indices = np.argsort(td)[topZ]
        elif type(topZ) is float:   # Return topics above a threshold.
            top_indices = [ z for z in np.argsort(td) if td[z] > topZ ]

        top_values = [ td[z] for z in top_indices ]
        
        topics = zip(top_indices, top_values)
        
        return topics
        
    def docs_in_topic(self, z, topD=None):
        """
        Returns a list of the topD documents most representative of topic z.
        
        Parameters
        ----------
        z : int
            A topic index.
        topD : int or float
            Number of prominent topics to return (int), or threshold (float).
            
        Returns
        -------
        documents : list
            List of (document, proportion) tuples.
        """    
        td = self.doc_topic[:, z]
        
        if topD is None:
            topD = td.shape[0]
        
        if type(topD) is int:   # Return a set number of documents.
            top_indices = np.argsort(td)[topD]
        elif type(topD) is float:   # Return documents above a threshold.
            top_indices = [ d for d in np.argsort(td) if td[d] > topD ]
        
        top_values = [ td[d] for d in top_indices ]
        top_idents = [ self.metadata[d] for d in top_indices ]
        
        documents = zip(top_idents, top_values)
        
        return documents


"""
To do: move testing of different types of attributes to writers testing
rather than graph testing; NetworkX can support any data structure as an
attribute but the graph file formats may not.
"""

import tethne.readers as rd
import tethne.networks as nt
import tethne.writers as wr
import tethne.utilities as util
import unittest
import networkx as nx
import sys

class TestDirectCitationGraph(unittest.TestCase):
    """
    Test the citations, internal_citations networks (assuming the reader
        is functioning) by checking both 'citation' and 'internal' networks
        each with either 'doi' or 'ayjid' node_ids and either no
        node attributes or 4 node attributes of 4 different variable
        types
    The sparse data file networks are built on is //testin/wos_citations.txt
    """
    def setUp(self):
        wos_data = rd.wos.parse('./testin/wos_citations.txt')
        meta_list = rd.wos.convert(wos_data)

        # ayjid with no attributes
        (self.ayjid_citations,
         self.ayjid_internal) = nt.papers.direct_citation(meta_list, 'ayjid')

        # doi with no attributes
        (self.doi_citations,
         self.doi_internal) = nt.papers.direct_citation(meta_list, 'doi')

        # int, string, list, dict
        self.node_attribs = ('date', 'atitle', 'aulast', 'citations')

        # ayjid with attribs
        (self.ayjid_cit_attrib,
         self.ayjid_int_attrib) = nt.papers.direct_citation(meta_list,
                                                  'ayjid',
                                                  *self.node_attribs)
        # doi with attribs
        (self.doi_cit_attrib,
         self.doi_int_attrib) = nt.papers.direct_citation(meta_list,
                                                'doi',
                                                *self.node_attribs)

    def test_ayjid(self):
        """
        Test of citations with ayjid as node_id and no node attributes

        If fails, something is wrong with the construction of ayjid's
        """
        # three papers in file each with author, year, journal information
        self.assertEqual(nx.number_of_nodes(self.ayjid_internal), 3)

        # ayjid of first paper's first reference does not match exactly
        # with the ayjid of the second paper
        self.assertEqual(nx.number_of_edges(self.ayjid_internal), 0)

        # since inexact match, 3 papers with 8 citations
        self.assertEqual(nx.number_of_nodes(self.ayjid_citations), 10)

        # one edge from paper to its citations
        self.assertEqual(nx.number_of_edges(self.ayjid_citations), 7)

    def test_doi(self):
        """
        Test of citations with doi as node_id and no node attributes

        If fails, something is wrong with the parsing of WOS files at
        either the DI field tag for papers or the DOI number in the CR
        field tag
        """
        # three papers in file, each with DI(=DOI) field tags
        self.assertEqual(nx.number_of_nodes(self.doi_internal), 3)

        # DOI does match exactly: first paper references second paper
        self.assertEqual(nx.number_of_edges(self.doi_internal), 1)

        # only 7 of the papers/references have DOI numbers
        self.assertEqual(nx.number_of_nodes(self.doi_citations), 7)

        # first paper has edges to its 3 citations, second has one to its
        # and third has one
        self.assertEqual(nx.number_of_edges(self.doi_citations), 5)

    def test_node_attribs(self):
        """
        Test adding of node attributes to 4 graphs: ayjid no attribs, ayjid
            with attribs, doi no attribs, and doi with attribs

        If fails, something is wrong with the assigning of node attributes
        from a meta_dict in the citations networks
        """
        # check that the attributes got added to each node
        graphs = [self.ayjid_cit_attrib, self.ayjid_int_attrib,
                  self.doi_cit_attrib, self.doi_int_attrib]
        for graph in graphs:
            for node in graph.nodes():
                node_attribs = sorted(graph.node[node])
                check_attribs = sorted(self.node_attribs)
                self.assertEqual(node_attribs, check_attribs)

    def test_edge_attribs(self):
        """
        Test adding of edge attributes indicating what date a paper is cited

        If fails, something is wrong with assigning the edge attribute from a
        paper to its citation indicating the date of citation
        """
        # attributes of first paper citing its first reference
        edge_attribs = self.doi_citations.edge \
                ['10.1007/s10646-013-1042-4'] \
                    ['10.1577/1548-8659(1993)122<0063:AQHAIF>2.3.CO;2']
        keys = edge_attribs.keys()
        self.assertIn('date', keys)
        self.assertEqual(edge_attribs['date'], 2013)

    def test_no_edge_attrib(self):
        """
        Test adding of edge attributes when citating paper's date is missing

        If the publication date is unknown (that is, PY field tag is missing
        from the WOS data file), the edge_attribute has value None; if
        fails that assumption is wrong
        """
        # attributes of third paper citing its only reference, PY=date key
        # missing
        edge_attribs = self.doi_citations.edge \
                ['10.1016/j.ecolind.2011.07.024']['10.1016/j.ecss.2008.08.010']
        keys = edge_attribs.keys()
        self.assertIn('date', keys)
        self.assertEqual(edge_attribs['date'], None)

    def tearDown(self):
        pass

class TestDirectCitation_DAG(unittest.TestCase):
    """
    Test the citations, internal_citations networks
    (assuming the reader is functioning)
    by checking both 'citation' and 'internal' networks
    each with either 'doi' or 'ayjid' node_ids and date as an attribute,

    This is the somewhat similar to the class TestDirectCitationGraph
    but it has extra tests to check if it is a DAG or not, and if the
    ancestors/descendants are populated correctly. The sparse data file networks
    are built on is "../testsuite/testin/citations_test.txt"

    It has 2 papers.
    """
    def setUp(self):

        wos_data = rd.wos.parse("../testsuite/testin/citations_test.txt")
        meta_list = rd.wos.convert(wos_data)

        # ayjid with no attributes
        (self.ayjid_citations,
         self.ayjid_internal) = nt.papers.direct_citation(meta_list, 'ayjid')

        # doi with no attributes
        (self.doi_citations,
         self.doi_internal) = nt.papers.direct_citation(meta_list, 'doi')


        # ayjid with attribs
        (self.ayjid_cit_attrib,
         self.ayjid_int_attrib) = nt.papers.direct_citation(meta_list,
                                                  'ayjid',
                                                  'date')
        # doi with attribs
        (self.doi_cit_attrib,
         self.doi_int_attrib) = nt.papers.direct_citation(meta_list,
                                                'doi',
                                                'date')

    def test_ayjid(self):
        """
        Test of citations with ayjid as node_id and no node attributes

        If fails, something is wrong with the construction of ayjid's
        """
        # three papers in file each with author, year, journal information
        self.assertEqual(nx.number_of_nodes(self.ayjid_internal), 2)

        # only 1 edge between 2 internal nodes
        self.assertEqual(nx.number_of_edges(self.ayjid_internal), 1)

        # 56
        self.assertEqual(nx.number_of_nodes(self.ayjid_citations), 56)

        # 65 edges
        self.assertEqual(nx.number_of_edges(self.ayjid_citations), 65)

    def test_doi(self):
        """
        Test of citations with doi as node_id and no node attributes

        If fails, something is wrong with the parsing of WOS files at
        either the DI field tag for papers or the DOI number in the CR
        field tag
        """
        # 2 internal papers
        self.assertEqual(nx.number_of_nodes(self.doi_internal), 2)

        # DOI does match exactly:
        self.assertEqual(nx.number_of_edges(self.doi_internal), 0)

        # only 19 of the papers/references have DOI numbers
        self.assertEqual(nx.number_of_nodes(self.doi_citations), 19)

        #22 edges between the nodes.
        self.assertEqual(nx.number_of_edges(self.doi_citations), 22)

    def is_citationnetwork_dag(self):
        """
        Testing if the citations graph is Directed Acyclic Graph or not.
        If it is false then the network will not be created and
        a Networkxerror will be thrown that it is not a DAG.
        """
        # Returns true if citations graph is DAG
        self.assert_(nx.is_directed_acyclic_graph(self.ayjid_cit_attrib),
                        "Citations Graph is not DAG")

    def is_internal_citationnetwork_dag(self):
        """
        Testing if Internal citations graph is Directed Acyclic Graph or not.
        If it is false then the network will not be created and an Networkxerror
        will be thrown that it is not a DAG.

        """
        # Returns true if citations graph is DAG
        self.assert_(nx.is_directed_acyclic_graph(self.ayjid_int_attrib),
                     "Internal Citations Graph is not DAG")


    def test_ancestors(self):
        """
        Testing the ancestors for a particular node which is specified
        in the input.It throws the error "Node not in the graph"
        if that node is not present.
        """

        #ESHELMAN is one of the citation of paper 1.
        c_ans = nx.ancestors(self.ayjid_citations,
                             'ESHELMAN LJ 1993 FDN GENETIC ALGORITH')
        #ALAMPORA G is a citation as well as a paper,
            #hence its a node in internal citations
        i_ans = nx.ancestors(self.ayjid_internal,
                             'ALAMPORA G 1999 INFORMATION SCIENCES')

    def test_descendants(self):
        """
        Testing the descendants of a particular node given in the  input.
        It throws the error "Node not in the graph" if that node is not present.
        """

        c_des = nx.descendants(self.ayjid_citations,
                               'ALAMPORA G 1999 INFORMATION SCIENCES')
        i_des = nx.descendants(self.ayjid_internal,
                               'WU Z 2012 NEUROCOMPUTING')

        # to check if error is raised if a node which
        # is not in the graph is called.

        # commented as of now
        # c_des_err = \
        # nx.descendants(self.ayjid_citations, \
        #     'ALAMPORA G 1999 addadaddsa SCIENCES')

        # self.assertRaises(NetworkXError, \
        # testsuite.networks_test.TestDirectCitation_DAG)

        #=======================================================================
        # try:
        #     c_des_err = nx.descendants(self.ayjid_citations, \
        #                   'ALAMPORA G 1999 addadaddsa SCIENCES')
        # except NetworkXError :
        #     self.fail('NetworkXerror thrown:', NetworkXError)
        #=======================================================================

    def tearDown(self):
        pass


class TestAuthorPapersGraph(unittest.TestCase):
    """
    Test the author_papers network
    Assumes reader is functioning

    The test file ./testin/wos_authors.txt contains two papers in WoS field tag
    format that are from the same author. One is fully capitalized and the
    other is not.

    We also test multiple authors and their link to the same paper
    """
    def setUp(self):
        wos_data = rd.wos.parse('./testin/wos_authors.txt')
        meta_list = rd.wos.convert(wos_data)
        self.no_attribs = nt.authors.author_papers(meta_list, 'doi')

        # int, string, list, dict
        node_attribs = ('date', 'atitle', 'aulast', 'citations')
        self.node_attribs = nt.authors.author_papers(meta_list, 'doi',
                                                *node_attribs)

    def test_case_sensitivity(self):
        """
        Fails if case sensitivity of author names matters
        This is particularly relevant for author_papers network because
        a string identifier is all we have to identify authors

        more of a reader test case
        """
        person_count = 0
        for node in self.no_attribs:
            if self.no_attribs.node[node]['type'] == 'person':
                person_count += 1

        self.assertEqual(person_count, 2)

    def test_no_attribs(self):
        # 2 authors; 2 papers
        self.assertEqual(nx.number_of_nodes(self.no_attribs), 4)

        # ADAMS has 2 papers, MCLEAN has 1 paper
        self.assertEqual(nx.number_of_edges(self.no_attribs), 3)

        # each node minimally has the type attribute distinguishing
        # paper nodes from person nodes
        expected_keys = ['type']
        for node in self.no_attribs.nodes(data=True):
            node_attribs = node[1]
            obtained_keys = node_attribs.keys()
            self.assertEqual(expected_keys, obtained_keys)

    def test_node_attribs(self):
        # 2 authors; 2 papers
        self.assertEqual(nx.number_of_nodes(self.no_attribs), 4)

        # ADAMS has 2 papers, MCLEAN has 1 paper
        self.assertEqual(nx.number_of_edges(self.no_attribs), 3)

        # 2 edges from author Adams S to his 2 papers
        # the following information is pulled from the test WoS file
        paper1_node_id = '10.1577/1548-8659(1993)122<0063:AQHAIF>2.3.CO;2'

        # paper 1 node attributes
        atitle1 = None
        aulast1 = ['ADAMS']
        date1 = 1993
        cr1_list = ['Adams S. M., 1990, AM FISHERIES SOC S',
                    ('ADAMS SM, 1985, J FISH BIOL, V26, P111,' +
                        'DOI 10.1111/j.1095-8649.1985.tb04248.x'),
                    'ADAMS SM, 1992, ECOTOX ENVIRON SAFE, V24, P347',
                    'Adams SM, 1990, AM FISHERIES SOC S, V8, P1']
        citations1 = []
        for cite in cr1_list:
            cite_dict = rd.wos.parse_cr(cite)
            citations1.append(cite_dict)

        # and those attributes in the expected form: a 'primitive' dictionary
        paper1_expected_attribs = {'atitle':atitle1,
                                   'aulast':aulast1,
                                   'date':date1,
                                   'citations':citations1,
                                   'type':'paper'}
        paper1_expected_keys = paper1_expected_attribs.keys()
        paper1_expected_attribs = util.subdict(paper1_expected_attribs,
                                                      paper1_expected_keys)

        # and for paper 2 nodes (note ADAMS S is an author for both papers)
        paper2_node_id = '10.1111/j.1095-8649.1985.tb04248.x'

        # and paper 2 node attributes
        atitle2 = None
        aulast2 = ['Adams', 'MCLEAN']
        date2 = 1985
        cr2_list = [('CR ADAMS SM, 1982, CAN J FISH AQUAT SCI, V39,' +
                        'P1175, DOI 10.1139/f82-155'),
                    ('ADAMS SM, 1982, T AM FISH SOC, V111, P549,' +
                        'DOI 10.1577/1548-8659(1982)111<549:EPILBU>2.0.CO;2'),
                    ('ALLEN JRM, 1982, J FISH BIOL, V21, P537,' +
                        'DOI 10.1111/j.1095-8649.1982.tb02858.x')]
        citations2 = []
        for cite in cr2_list:
            cite_dict = rd.wos.parse_cr(cite)
            citations2.append(cite_dict)

        # and those attributes in the expected form: a 'primitive' dictionary
        paper2_expected_attribs = {'atitle':atitle2,
                                   'aulast':aulast2,
                                   'date':date2,
                                   'citations':citations2,
                                   'type':'paper'}
        paper2_expected_keys = paper2_expected_attribs.keys()
        paper2_expected_attribs = util.subdict(paper2_expected_attribs,
                                                      paper2_expected_keys)



        # verify the contents of paper 1
        paper1_obtained_attribs = self.node_attribs.node[paper1_node_id]
        self.assertItemsEqual(paper1_expected_attribs,
                              paper1_obtained_attribs)

        # and paper 2
        paper2_obtained_attribs = self.node_attribs.node[paper2_node_id]
        self.assertItemsEqual(paper2_expected_attribs,
                              paper2_obtained_attribs)

    def tearDown(self):
        pass


class TestCoauthorsGraph(unittest.TestCase):
    """
    Test the coauthors network, assumes reader is functioning
    The file has 2 papers in which 2 pairs of authors are co-authors in both.
    The edges attributes should contain a list which identifies each paper
    It should be a Normal graph and not a MultiGraph.
    """

    def setUp(self):

        wos_data =rd.wos.parse("../testsuite/testin/coauthors_2_recs.txt")
        meta_list = rd.wos.convert(wos_data)

        # There is no concept of user input threshold testcases as the authors \
        # will be mapped with their respective co-authors anyways.
        self.coauthors_noattribs  = nt.authors.coauthors(meta_list,)
        self.coauthors_one_attribs  = nt.authors.coauthors \
                            (meta_list,'ayjid')
        self.coauthors_three_attribs  = nt.authors.coauthors \
                            (meta_list,'ayjid','jtitle','date')

    # When no attribs are provided by the user.
    def test_coauthors_no_attribs(self):
        # 9 nodes: one for each author and his co-author
        self.assertEqual(nx.number_of_nodes(self.coauthors_noattribs), 9)
        # 26 edges between them
        self.assertEqual(nx.number_of_edges(self.coauthors_noattribs), 26)
        obtained_edge_attribs_dict = nx.get_edge_attributes \
                            (self.coauthors_noattribs,"")
        expected_edge_attribs_dict = {}
        self.assertDictEqual(expected_edge_attribs_dict, \
                        obtained_edge_attribs_dict, "Edge Attribs are equal")

    # When no attribs are provided by the user.
    def test_coauthors_one_attribs(self):
        # 9 nodes: one for each author and his co-author
        self.assertEqual(nx.number_of_nodes(self.coauthors_one_attribs),9)
        # 26 edges between them
        self.assertEqual(nx.number_of_edges(self.coauthors_one_attribs),26)
        obtained_edge_attribs_dict = nx.get_edge_attributes \
                (self.coauthors_one_attribs, "ayjid")
        expected_edge_attribs_dict = \
        {   ('JIN M', 'GARMENDIA L'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('IZAGIRRE U', 'JIN M'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('IZAGIRRE U', 'SOTO M'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('MARIGOMEZ I', 'ORBEA A'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('ORBEA A', 'GARMENDIA L'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('JIN M', 'SOTO M'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('CAJARAVILLE M', 'SOTO M'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('IZAGIRRE U', 'MARIGOMEZ I'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('IZAGIRRE U', 'CAJARAVILLE M'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('IZAGIRRE U', 'GARMENDIA L'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('TONG X', 'CAJARAVILLE M'):
                    ('HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT'),
            ('CAJARAVILLE M', 'MARIGOMEZ I'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('JIN M', 'TONG X'):
                    'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT',
            ('JIN M', 'CAJARAVILLE M'): ['MARIGOMEZ I 2013 ECOTOXICOLOGY',
             'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT'],
            ('ORBEA A', 'SOTO M'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('CAJARAVILLE M', 'GARMENDIA L'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('CAJARAVILLE M', 'ORBEA A'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('JIN M', 'ORBEA A'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('SOTO M', 'GARMENDIA L'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('HAN D', 'JIN M'): \
                'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT',
            ('HAN D', 'CAJARAVILLE M'): \
                'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT',
            ('MARIGOMEZ I', 'GARMENDIA L'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('HAN D', 'TONG X'): \
                'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT',
            ('IZAGIRRE U', 'ORBEA A'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('JIN M', 'MARIGOMEZ I'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY',
            ('MARIGOMEZ I', 'SOTO M'): 'MARIGOMEZ I 2013 ECOTOXICOLOGY'
        }


        self.assertDictEqual(expected_edge_attribs_dict,\
                             obtained_edge_attribs_dict)


        # When three attribs are provided by the user.
    def test_coauthors_three_attribs(self):
        # 9 nodes: one for each author and his co-author
        self.assertEqual(nx.number_of_nodes(self.coauthors_three_attribs),9)
        # 26 edges between them
        self.assertEqual(nx.number_of_edges(self.coauthors_three_attribs),26)
        obtained_edge_attribs_list = \
                        self.coauthors_three_attribs.edges(data=True)
        #Need to properly format them in 80Char limit.
        expected_edge_attribs_list = \
        [('IZAGIRRE U', 'JIN M', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('IZAGIRRE U', 'CAJARAVILLE M', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('IZAGIRRE U', 'MARIGOMEZ I', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('IZAGIRRE U', 'ORBEA A', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('IZAGIRRE U', 'SOTO M', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('IZAGIRRE U', 'GARMENDIA L', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('HAN D', 'CAJARAVILLE M', {'date': 2013, 'ayjid': 'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT', 'jtitle': 'ENVIRONMENTAL MONITORING AND ASSESSMENT'}), ('HAN D', 'JIN M', {'date': 2013, 'ayjid': 'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT', 'jtitle': 'ENVIRONMENTAL MONITORING AND ASSESSMENT'}), ('HAN D', 'TONG X', {'date': 2013, 'ayjid': 'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT', 'jtitle': 'ENVIRONMENTAL MONITORING AND ASSESSMENT'}), ('JIN M', 'TONG X', {'date': 2013, 'ayjid': 'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT', 'jtitle': 'ENVIRONMENTAL MONITORING AND ASSESSMENT'}), ('JIN M', 'CAJARAVILLE M', {'date': [2013, 2013], 'jtitle': ['ECOTOXICOLOGY', 'ENVIRONMENTAL MONITORING AND ASSESSMENT'], 'ayjid': ['MARIGOMEZ I 2013 ECOTOXICOLOGY', 'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT']}), ('JIN M', 'MARIGOMEZ I', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('JIN M', 'ORBEA A', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('JIN M', 'SOTO M', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('JIN M', 'GARMENDIA L', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('TONG X', 'CAJARAVILLE M', {'date': 2013, 'ayjid': 'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT', 'jtitle': 'ENVIRONMENTAL MONITORING AND ASSESSMENT'}), ('CAJARAVILLE M', 'MARIGOMEZ I', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('CAJARAVILLE M', 'ORBEA A', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('CAJARAVILLE M', 'SOTO M', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('CAJARAVILLE M', 'GARMENDIA L', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('MARIGOMEZ I', 'ORBEA A', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('MARIGOMEZ I', 'SOTO M', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('MARIGOMEZ I', 'GARMENDIA L', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('ORBEA A', 'SOTO M', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('ORBEA A', 'GARMENDIA L', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'}), ('SOTO M', 'GARMENDIA L', {'date': 2013, 'ayjid': 'MARIGOMEZ I 2013 ECOTOXICOLOGY', 'jtitle': 'ECOTOXICOLOGY'})]

        self.assertListEqual(expected_edge_attribs_list,obtained_edge_attribs_list)


    def tearDown(self):
            pass



class TestBiblioGraph(unittest.TestCase):
    """
    Test the bibliographic_coupling network; assumes reader is functioning

    The viewer will note that the input file has 4 papers trimmed down
    and modified from the sample //docs/savedrecs.txt.

    The references
    were constructed so that the first paper shares two citations with the
    second paper, one (of the two shared between first and second) citation
    with the third paper, and zero citations with the fourth paper.
    """
    def setUp(self):
        wos_data = rd.wos.parse('./testin/wos_biblio.txt')
        wos_meta = rd.wos.convert(wos_data)
        self.ayjid_zero = nt.papers.bibliographic_coupling(wos_meta,
                                                 'ayjid',
                                                 0,
                                                 'ayjid')
        self.ayjid_one = nt.papers.bibliographic_coupling(wos_meta,
                                               'ayjid',
                                               1,
                                               'ayjid')
        self.ayjid_two = nt.papers.bibliographic_coupling(wos_meta,
                                               'ayjid',
                                               2,
                                               'ayjid')
        self.ayjid_attribs = nt.papers.bibliographic_coupling(wos_meta,
                                                   'ayjid',
                                                   1,
                                                   'ayjid',
                                                   'atitle',
                                                   'aulast',
                                                   'date',
                                                   'citations')
        self.weighted = nt.papers.bibliographic_coupling(wos_meta,
                                               'ayjid',
                                               0.5,
                                               'ayjid',
                                               weighted=True)

        # define a separate file for the missing citations test case
        # with the same data as the other test file with the exception
        # that the fourth paper is missing its citation record
        wos_cite = rd.wos.parse('./testin/wos_biblio_cite.txt')
        doc_list_cite = rd.wos.convert(wos_cite)
        self.missing_citations_zero = \
                nt.papers.bibliographic_coupling(doc_list_cite,
                                                             'ayjid',
                                                             0,
                                                             'ayjid')

        self.missing_citations_one = \
                    nt.papers.bibliographic_coupling(doc_list_cite,
                                                            'ayjid',
                                                            1,
                                                            'ayjid')

    def test_ayjid_zero(self):
        """
        Every paper shares at least 0 references with every other
        paper. This makes a complete graph.
        """
        # 4 papers
        self.assertEqual(nx.number_of_nodes(self.ayjid_zero), 4)
        # 4 choose 2 = 6 edges
        self.assertEqual(nx.number_of_edges(self.ayjid_zero), 6)

    def test_ayjid_one(self):
        """
        With threshold greater than 0, two papers must share at least 1
        reference in common. Test this when threshold is 1.
        """
        self.assertEqual(nx.number_of_nodes(self.ayjid_one), 3)
        # first paper shares 2 >= 1 references with second, 1 >= 1 references
        # with third, and the second and third share 1 >= 1 references
        self.assertEqual(nx.number_of_edges(self.ayjid_one), 3)



    def test_ayjid_two(self):
        """
        With threshold greater than 0, two papers must share at least 1
        reference in common. Test this when threshold is 2.
        """
        self.assertEqual(nx.number_of_nodes(self.ayjid_two), 2)
        # first paper shares 2 >= 2 references with second
        self.assertEqual(nx.number_of_edges(self.ayjid_two), 1)

    def test_unweighted(self):
        """
        Unweighted similarities.
        """
        edges = self.ayjid_one.edges(data=True)
        self.assertEqual(edges[0][2]['similarity'], 2)
        self.assertEqual(edges[1][2]['similarity'], 1)
        self.assertEqual(edges[2][2]['similarity'], 1)

    def test_weighted(self):
        """
        Weighted similarities.
        """
        edges = self.weighted.edges(data=True)
        self.assertEqual(edges[0][2]['similarity'], 1.0)
        self.assertEqual(edges[1][2]['similarity'], 0.5)
        self.assertEqual(edges[2][2]['similarity'], 0.5)

    def test_attribs(self):
        """
        Test the addition of four node attributes of different types:
            atitle - a string
            date - an int
            aulast - a list
            citations - a dictionary
        Presumably a double may be added to a network just as easily as an
        int, but we have no doubles to test in our standard meta_dict
        """
        # paper 1 meta data
        node1_atitle = ('Marine ecosystem health status assessment' +
                        ' through integrative biomarker indices:' +
                        ' a comparative study after the Prestige oil' +
                        ' spill "Mussel Watch"')
        node1_date = 2013
        node1_aulast = ['Marigomez', 'Garmendia', 'Soto', 'Orbea',
                        'Izagirre', 'Cajaraville']
        for i in xrange(len(node1_aulast)):
            node1_aulast[i] = node1_aulast[i].upper().strip()
        node1_cr_list = [('ADAMS SM, 1993, T AM FISH SOC, V122, P63, DOI' +
                          ' 10.1577/1548-8659(1993)122<0063:AQHAIF>2.3.CO;2'),
                         ('Beliaeff B, 2002, ENVIRON TOXICOL CHEM, V21, ' +
                          'P1316, DOI 10.1897/1551-5028(2002)021<1316:' +
                          'IBRAUT>2.0.CO;2')]
        node1_citations = []
        for cite in node1_cr_list:
            node1_citations.append(rd.wos.parse_cr(cite))
        node1_expected_attribs = {'atitle':node1_atitle,
                                  'date':node1_date,
                                  'aulast':node1_aulast,
                                  'citations':node1_citations}
        node1_ayjid = 'MARIGOMEZ I 2013 ECOTOXICOLOGY'

        # paper 2 meta data
        node2_atitle = ('Evaluation of organic contamination in urban ' +
                        'groundwater surrounding a municipal landfill, ' +
                        'Zhoukou, China')
        node2_date = 2013
        node2_aulast = ['Han', 'Tong', 'Jin', 'Hepburn', 'Tong', 'Song']
        for i in xrange(len(node2_aulast)):
            node2_aulast[i] = node2_aulast[i].upper().strip()
        node2_cr_list = [('ADAMS SM, 1993, T AM FISH SOC, V122, P63, ' +
                          'DOI 10.1577/1548-8659(1993)122<0063:AQHAIF>' +
                          '2.3.CO;2'),
                         ('Beliaeff B, 2002, ENVIRON TOXICOL CHEM, V21, ' +
                          'P1316, DOI 10.1897/1551-5028(2002)021<1316:' +
                          'IBRAUT>2.0.CO;2')]
        node2_citations = []
        for cite in node2_cr_list:
            node2_citations.append(rd.wos.parse_cr(cite))
        node2_expected_attribs = {'atitle':node2_atitle,
                                  'date':node2_date,
                                  'aulast':node2_aulast,
                                  'citations':node2_citations}
        node2_ayjid = 'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT'

        # paper 3 meta data
        node3_atitle = ('Multilevel governance and management of shared ' +
                        'stocks with integrated markets: The European ' +
                        'anchovy case')
        node3_date = 2013
        node3_aulast = ['Mulazzani', 'Curtin', 'Andres', 'Malorgio']
        for i in xrange(len(node3_aulast)):
            node3_aulast[i] = node3_aulast[i].upper().strip()
        node3_cr_list = [('AdriaMed, GEN OUTL MAR CAPT FI'),
                         ('Beliaeff B, 2002, ENVIRON TOXICOL CHEM, V21, ' +
                          'P1316, DOI 10.1897/1551-5028(2002)021' +
                          '<1316:IBRAUT>2.0.CO;2')]
        node3_citations = []
        for cite in node3_cr_list:
            node3_citations.append(rd.wos.parse_cr(cite))
        node3_expected_attribs = {'atitle':node3_atitle,
                                  'date':node3_date,
                                  'aulast':node3_aulast,
                                  'citations':node3_citations}
        node3_ayjid = 'MULAZZANI L 2013 MARINE POLICY'

    def test_missing_citations_zero(self):
        """
        if fails network is not robust to papers missing citation data
        test for threshold = 0
        """
        # 4 papers
        self.assertEqual(nx.number_of_nodes(self.missing_citations_zero), 4)
        # 4 choose 2 = 6 edges
        self.assertEqual(nx.number_of_edges(self.missing_citations_zero), 6)


    def test_missing_citations_one(self):
        """
        if fails network is not robust to papers missing citation data
        test for threshold = 1
        """
        # 3 papers
        self.assertEqual(nx.number_of_nodes(self.missing_citations_one), 3)
        # first paper shares 2 >= 1 references with second, 1 >= 1 references
        # with third, and the second and third share 1 >= 1 references and
        # fourth constructed to have no citations
        self.assertEqual(nx.number_of_edges(self.missing_citations_one), 3)

    def tearDown(self):
        pass


class TestAuthorCouplingGraph(unittest.TestCase):
    """
    Test the author_coupling network
    Assumes reader is functioning

    the //testin/wos_author_coupling.txt file has been constructed with
    the following properties:
        the first paper shares 2 authors with the second and 1 with the third
        the second paper shares 1 author with the third
        the fourth paper shares 0 authors with any other paper
    """
    def setUp(self):
        # read data from docs folder
        wos_data = rd.wos.parse('./testin/wos_author_coupling.txt')
        meta_list = rd.wos.convert(wos_data)

        self.ayjid_zero = nt.papers.author_coupling(meta_list,
                                                0,
                                                'ayjid')

        self.ayjid_one = nt.papers.author_coupling(meta_list,
                                               1,
                                               'ayjid')

        self.ayjid_two = nt.papers.author_coupling(meta_list,
                                               2,
                                               'ayjid')

        self.ayjid_attribs = nt.papers.author_coupling(meta_list,
                                                   1,
                                                   'ayjid',
                                                   'atitle',
                                                   'date')
    def test_ayjid_zero(self):
        # with four papers there are 4 nodes
        self.assertEqual(nx.number_of_nodes(self.ayjid_zero), 4)

        # every paper shares at least 0 >= 0 authors with other papers
        # so we expect 4 choose 2 = 6 edges
        self.assertEqual(nx.number_of_edges(self.ayjid_zero), 6)

    def test_ayjid_one(self):
        # four nodes: one for each paper
        self.assertEqual(nx.number_of_nodes(self.ayjid_one), 4)

        # as noted in doc string, only papers 1 2 and 3 share one author
        # among them; we expect 3 edges
        self.assertEqual(nx.number_of_edges(self.ayjid_one), 3)

    def test_ayjid_two(self):
        # four nodes: one for each paper
        self.assertEqual(nx.number_of_nodes(self.ayjid_two), 4)

        # as noted in the doc string, one papers 1 and 2 share two authors
        # we expect 1 edge
        self.assertEqual(nx.number_of_edges(self.ayjid_two), 1)

    def test_ayjid_attribs(self):
        # paper 1 meta data
        node1_atitle = ('Marine ecosystem health status assessment' +
                        'through integrative biomarker indices: a' +
                        'comparative study after the Prestige oil' +
                        'spill "Mussel Watch"')
        node1_date = 2013
        node1_expected_attribs = {'atitle':node1_atitle,
                                  'date':node1_date}
        node1_ayjid = 'MARIGOMEZ I 2013 ECOTOXICOLOGY'

        # paper 2 meta data
        node2_atitle = ('Evaluation of organic contamination in urban' +
                        'groundwater surrounding a municipal landfill,' +
                        'Zhoukou, China')
        node2_date = 2013
        node2_expected_attribs = {'atitle':node2_atitle,
                                  'date':node2_date}
        node2_ayjid = 'HAN D 2013 ENVIRONMENTAL MONITORING AND ASSESSMENT'

        # paper 3 meta data
        node3_atitle = ('Multilevel governance and management of shared' +
                        'stocks with integrated markets: The European' +
                        'anchovy case')
        node3_date = 2013
        node3_expected_attribs = {'atitle':node3_atitle,
                                  'date':node3_date}
        node3_ayjid = 'MULAZZANI L 2013 MARINE POLICY'

        # paper 4 meta data
        node4_atitle = ('Application of multiple geochemical markers to' +
                        'investigate organic pollution in a dynamic coastal' +
                        'zone')
        node4_date = 2013
        node4_expected_attribs = {'atitle':node4_atitle,
                                  'date':node4_date}
        node4_ayjid = 'LIU L 2013 ENVIRONMENTAL TOXICOLOGY AND CHEMISTRY'

        # test attribute expectations
        node_id_list = [node1_ayjid, node2_ayjid, node3_ayjid, node4_ayjid]
        node_attrib_list = [node1_expected_attribs, node2_expected_attribs,
                            node3_expected_attribs, node4_expected_attribs]
        for i in xrange(len(node_id_list)):
            node = node_id_list[i]
            obtained_attribs = self.ayjid_attribs.node[node]
            expected_attribs = node_attrib_list[i]
            self.assertItemsEqual(expected_attribs, obtained_attribs)

    def tearDown(self):
        pass

class TestAuthorCocitation(unittest.TestCase):
    """
    Test the author cocitation (analysis) network
    Assumes reader is functioning

    testfile is
    "/Users/ramki/tethne/tethne/testsuite/testin/cocitations_test_full.txt"


     """
    def setUp(self):

        wos_data =rd.wos.parse \
                ("../testsuite/testin/cocitations_test_full.txt")
        meta_list = rd.wos.convert(wos_data)

        #No need to check edge / node attribs as there are none.

        self.cocitations_zero  = nt.authors.author_cocitation(meta_list, 0)

        self.cocitations_one  = nt.authors.author_cocitation(meta_list, 1)

        self.cocitations_two  = nt.authors.author_cocitation(meta_list, 2)

        self.cocitations_three = nt.authors.author_cocitation(meta_list, 3)


    #When the Input threshold is
    def test_cocitations_zero(self):
        # 249 nodes: one for each author and his co-author
        self.assertEqual(nx.number_of_nodes(self.cocitations_zero), 235)
        # as noted in doc string, 12 edges between them
        self.assertEqual  (nx.number_of_edges(self.cocitations_zero), 4401)
    pass


    def test_cocitations_one(self):
        # 249 nodes: one for each author and his co-author
        self.assertEqual  (nx.number_of_nodes(self.cocitations_one), 235)
        # as noted in doc string, 12 edges between them
        self.assertEqual  (nx.number_of_edges(self.cocitations_one), 4401)
    pass


    def test_cocitations_two(self):
        # 19 nodes: one for each author, one for each institution
        self.assertEqual  (nx.number_of_nodes(self.cocitations_two), 13)
        # as noted in doc string, 12 edges between them
        self.assertEqual  (nx.number_of_edges(self.cocitations_two), 35)
    pass


    def test_cocitations_three(self):
        # 19 nodes: one for each author, one for each institution
        self.assertEqual (nx.number_of_nodes(self.cocitations_three), 2)
        # as noted in doc string, 12 edges between them
        self.assertEqual (nx.number_of_edges(self.cocitations_three), 1)
    pass


class TestAuthorInstitution(unittest.TestCase):
    """
    Test the author_institutions network
    Assumes reader is functioning

    "/Users/ramki/tethne/tethne/testsuite/testin/authorinstitutions_test.txt"
    file has been constructed with
    the following properties:
        - the first paper has 6 authors and 6 institutions ,
        where one author Wu,ZD shares affiliations with
        2 other different institutions and authors.
        - the second paper has 3 authors with 2 institutions,
        where 2 authors share same affliated institution.
        - the third paper has no 'institutions' key field ,
        hence no author-institutions sharing.
        - the fourth paper has 1 author and 1 institution ,
        with no shared institutions.

    """


    # There are 10 authors with 9 institutions, as given in the doc string.

    #Nodes:
    #-------
    # As the networks is built between 10 authors and 9 institutions,
    # there should be 19 nodes (10+9). - test 1 is checked in other tests.
    # Test the node attributes - (value="author / institutions")
    # It should match with number of nodes and ----
    #--- distinguished author and institutions count
    #(value = "authors" count=10,value = "Institutions" count= 9,total19)-test 2

    #Edges:
    #------
    # To test the edges, there is no concept of providing input 'threshold'
    # from the user.
    # 12 edges ( 10 + 2 ) between the 10 authors and 9 institutions
    # where 2 authors are affliated to 2 different institutions   - test 3
    # check the edge attributes - 'Date' is an edge attribute  - test 4
    # check the edge attributes - no edge attributes are provided - test 5


    def setUp(self):
        wos_data = rd.wos.parse("./testin/authorinstitutions_test.txt")
        meta_list = rd.wos.convert(wos_data)

        self.node_attribs_check = nt.authors.author_institution(meta_list)

        self.shared_institutions  = nt.authors.author_institution(meta_list)

        self.edge_attribs_zero = nt.authors.author_institution(meta_list)

        self.edge_attribs_one = nt.authors.author_institution(meta_list,'date')




    def test_shared_institutions(self):
        # 19 nodes: one for each author, one for each institution
        self.assertEqual(nx.number_of_nodes(self.shared_institutions), \
                         19, "Nodes Equal")
        # as noted in doc string, 12 edges between them
        self.assertEqual(nx.number_of_edges(self.shared_institutions),  \
                         12, "Edges Equal")


    def test_node_attribs_check(self):

        obtained_node_attribs_dict = nx.get_node_attributes \
                    (self.node_attribs_check,"type")

        expected_node_attribs_dict = {'Natl Chung Cheng Univ': 'institution', \
            'Victoria Univ': 'institution',
         'ZHANG, YC': 'author', 'ACAMPORA, G': 'author', \
             'Univ Sci & Technol China': 'institution',
         'Huang, TCK': 'author', 'Eindhoven Univ Technol': 'institution',
         'XU, GD': 'author', \
             'Inst Sci & Technol Informat Zhejiang Prov': 'institution',
         'WU, ZD': 'author', 'Wenzhou Univ': 'institution', \
             'Northwestern Polytech Univ': 'institution',
          'LU, CL': 'author', 'CHEN, EH': 'author', \
              'VITIELLO, A': 'author',
          'Univ Salerno': 'institution', 'ZHANG, H': 'author',
          'LOIA, V': 'author', 'Univ Technol Sydney': 'institution'}

        self.assertDictEqual(obtained_node_attribs_dict, \
                             expected_node_attribs_dict, \
                             "Node attribs of Author institutions Equal")

    def test_edge_attribs_zero(self):

        obtained_edge_attribs_dict = \
        nx.get_edge_attributes(self.edge_attribs_zero,"")

        expected_edge_attribs_dict = {}

        self.assertDictEqual(expected_edge_attribs_dict, \
                            obtained_edge_attribs_dict, \
                            "Edge Attribs are equal")


    def test_edge_attribs_one(self):

       obtained_edge_attribs_dict = \
            nx.get_edge_attributes(self.edge_attribs_one, "date")

       expected_edge_attribs_dict = {('Wenzhou Univ', 'LU, CL'): 2012, \
            ('ACAMPORA, G', 'Eindhoven Univ Technol'): 1999,
            ('Univ Technol Sydney', 'XU, GD'): 2012,\
            ('Northwestern Polytech Univ', 'LU, CL'): 2012, \
            ('Univ Salerno', 'LOIA, V'): 1999, \
            ('ZHANG, YC', 'Victoria Univ'): 2012,\
            ('Inst Sci & Technol Informat Zhejiang Prov', 'ZHANG, H'): 2012,\
            ('WU, ZD', 'Wenzhou Univ'): 2012,
            ('Natl Chung Cheng Univ', 'Huang, TCK'): 2013, \
            ('Univ Sci & Technol China', 'CHEN, EH'): 2012, \
            ('WU, ZD', 'Univ Sci & Technol China'): 2012, \
            ('VITIELLO, A', 'Univ Salerno'): 1999}

       self.assertDictEqual(expected_edge_attribs_dict, \
                            obtained_edge_attribs_dict, \
                            "Edge Attribs are equal")


    def tearDown(self):
         pass


class TestAuthorCoinstitution(unittest.TestCase):
    """
    Test the author_coinstitution network
    Assumes reader is functioning

    the //testin/authorinstitution_testrecords.txt file is constructed with
    the following properties:
    the first paper has 6 authors and 6 institutions , \
    where one author Wu,ZD shares affiliations with  \
    2 other different institutions and authors.
    the second paper has 3 authors with 2 institutions, \
        where 2 authors share same affliated institution.
    the third paper has no 'institutions' key field \
        hence no author-institutions sharing.
    the fourth paper has 1 author and 1 institution \
        with no shared institutions.

    """

    # There are 10 authors with 9 institutions, as given in the doc string.

    #Nodes:
    #-------
    # As the networks is built only between 10 authors ,
    # there should be 10 nodes. - test 1 satisfied in all other tests
    # Test the node attributes of the author-(value="author") -test 2

    #Edges:
    #------
    # To test the edges, we need to give various options
    # for the threshold of shared institutions between them.
    # Threshold =0,  45 edges between the 10 authors(nodes) N*(N-1) /2  - test 3
    # Threshold =1,  3 edges between the authors(nodes)
    # i.e 3 authors have 1 institution shared among them - test 4
    # Threshold =2,  no edges between the authors(nodes)
    # i.e no authors have 2 institutions shared among them - test 5
    # No edge attributes, hence no tests required.


    def setUp(self):
        wos_data = rd.wos.parse("./testin/authorinstitutions_test.txt")
        meta_list = rd.wos.convert(wos_data)

        self.node_attribs_check = \
                nt.authors.author_coinstitution(meta_list, 1) # test 2

        self.shared_institutions_zero = \
                nt.authors.author_coinstitution(meta_list, 0)  # test 3

        self.shared_institutions_one = \
                nt.authors.author_coinstitution(meta_list, 1)  # test 4

        self.shared_institutions_two = \
                nt.authors.author_coinstitution(meta_list, 2)  # test 5



    def test_shared_institutions_zero(self):
        # 10 nodes: one for each author
        self.assertEqual(nx.number_of_nodes(self.shared_institutions_zero), 10)
        # as noted in doc string, 45 edges between them
        self.assertEqual(nx.number_of_edges(self.shared_institutions_zero), 45)


    def test_shared_institutions_one(self):
        # 5 nodes: #63128668
        self.assertEqual(nx.number_of_nodes(self.shared_institutions_one), 5)
        # as noted in doc string, 3 edges between them
        self.assertEqual(nx.number_of_edges(self.shared_institutions_one), 3)

    def test_shared_institutions_two(self):
        # 0 nodes: #63128668
        self.assertEqual(nx.number_of_nodes(self.shared_institutions_two), 0)
        # as noted in doc string, 0 edges between them
        self.assertEqual(nx.number_of_edges(self.shared_institutions_two), 0)

    def test_node_attribs_check(self):

        obtained_node_attribs_dict = \
            nx.get_node_attributes(self.node_attribs_check,"type")

        expected_node_attribs_dict = \
            {'WU, ZD': 'author', 'VITIELLO, A': 'author',
                'LU, CL': 'author', 'LOIA, V': 'author', 'CHEN, EH': 'author'}

        self.assertDictEqual \
        (expected_node_attribs_dict, \
                    obtained_node_attribs_dict, " Node Attribs are not same")

    def tearDown(self):
         pass


class TestCocitation(unittest.TestCase):
    """
    Test the cocitations network
    Assumes reader is functioning

    The ./testin/cocitations_test_2recs.txt file has been constructed with
    The following properties:

    There are 2 article papers in the file

    number of citations in paper1 = n1(8 in this paper)
    number of citations in paper1 = n2(9 in this paper)

    They share 4 common citations among them ( threshold =2)

    """

    # There are 2 papers with n1+n2 cited papers ( 8+9 = 17) in this case

    #Nodes:
    #-------
    # As the networks is built between cited papers , there should be 17 nodes
    # As of now there are node attributes of the paper to be tested


    #Edges:
    #------
    # To test the edges, we need to give various options for the 'threshold'
    # value in input. of cocitations  between them.
    # Threshold =0,  58 edges between the cited papers
    # Threshold =1,  58 edges between the cited papers
    # Threshold =2,  6 edges between the cited papers
    # No edge attributes as of now, hence no tests required.


    def setUp(self):

        wos_data = rd.wos.parse("./testin/cocitations_test_2recs.txt")
        meta_list = rd.wos.convert(wos_data)
        self.cocitations_zero = nt.papers.cocitation(meta_list, 0) #test 1
        self.cocitations_one = nt.papers.cocitation(meta_list, 1)  # test 2
        self.cocitations_two = nt.papers.cocitation(meta_list, 2)  # test 5




    def test_cocitations_zero(self):
        # 8+9 - 4common nodes :
        self.assertEqual(nx.number_of_nodes(self.cocitations_zero), 13)
        self.assertEqual(nx.number_of_edges(self.cocitations_zero), 58)

        obtained = set(self.cocitations_zero.edges())
        expected = set([('YU HL 2007 STOCH ENV RES RISK A', 'ZOU CK 2011 TECHNOMETRICS'), ('YU HL 2007 STOCH ENV RES RISK A', 'FILIPPIDIS A 1992 FUEL'), ('YU HL 2007 STOCH ENV RES RISK A', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('YU HL 2007 STOCH ENV RES RISK A', 'LANG CL 2012 NAV RES LOG'), ('YU HL 2007 STOCH ENV RES RISK A', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('YU HL 2007 STOCH ENV RES RISK A', 'VATALIS KI 2006 ENVIRON MANAGE'), ('YU HL 2007 STOCH ENV RES RISK A', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE'), ('ZOU CK 2011 TECHNOMETRICS', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('ZOU CK 2011 TECHNOMETRICS', 'FILIPPIDIS A 1992 FUEL'), ('ZOU CK 2011 TECHNOMETRICS', 'LANG CL 2012 NAV RES LOG'), ('ZOU CK 2011 TECHNOMETRICS', 'EFRON B 1993 INTRO BOOTSTRAP'), ('ZOU CK 2011 TECHNOMETRICS', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('ZOU CK 2011 TECHNOMETRICS', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE'), ('ZOU CK 2011 TECHNOMETRICS', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('ZOU CK 2011 TECHNOMETRICS', 'FRANK A 2010 UCI MACHINE LEARNING'), ('ZOU CK 2011 TECHNOMETRICS', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('ZOU CK 2011 TECHNOMETRICS', 'VATALIS KI 2006 ENVIRON MANAGE'), ('ZOU CK 2011 TECHNOMETRICS', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('FILIPPIDIS A 1992 FUEL', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('FILIPPIDIS A 1992 FUEL', 'LANG CL 2012 NAV RES LOG'), ('FILIPPIDIS A 1992 FUEL', 'EFRON B 1993 INTRO BOOTSTRAP'), ('FILIPPIDIS A 1992 FUEL', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('FILIPPIDIS A 1992 FUEL', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE'), ('FILIPPIDIS A 1992 FUEL', 'FRANK A 2010 UCI MACHINE LEARNING'), ('FILIPPIDIS A 1992 FUEL', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('FILIPPIDIS A 1992 FUEL', 'VATALIS KI 2006 ENVIRON MANAGE'), ('FILIPPIDIS A 1992 FUEL', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('FILIPPIDIS A 1992 FUEL', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('FRANK A 2010 UCI MACHINE LEARNING', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('FRANK A 2010 UCI MACHINE LEARNING', 'EFRON B 1993 INTRO BOOTSTRAP'), ('FRANK A 2010 UCI MACHINE LEARNING', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('FRANK A 2010 UCI MACHINE LEARNING', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('FRANK A 2010 UCI MACHINE LEARNING', 'LANG CL 2012 NAV RES LOG'), ('FRANK A 2010 UCI MACHINE LEARNING', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('EFRON B 1993 INTRO BOOTSTRAP', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('EFRON B 1993 INTRO BOOTSTRAP', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('EFRON B 1993 INTRO BOOTSTRAP', 'LANG CL 2012 NAV RES LOG'), ('EFRON B 1993 INTRO BOOTSTRAP', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('EFRON B 1993 INTRO BOOTSTRAP', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'LANG CL 2012 NAV RES LOG'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'VATALIS KI 2006 ENVIRON MANAGE'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('ADRIANO DC 2001 TRACE ELEMENTS TERRE', 'LANG CL 2012 NAV RES LOG'), ('ADRIANO DC 2001 TRACE ELEMENTS TERRE', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('ADRIANO DC 2001 TRACE ELEMENTS TERRE', 'VATALIS KI 2006 ENVIRON MANAGE'), ('DAS N 2009 QUAL TECHNOL QUANT M', 'LANG CL 2012 NAV RES LOG'), ('DAS N 2009 QUAL TECHNOL QUANT M', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('DAS N 2009 QUAL TECHNOL QUANT M', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('LANG CL 2012 NAV RES LOG', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('LANG CL 2012 NAV RES LOG', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('LANG CL 2012 NAV RES LOG', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('LANG CL 2012 NAV RES LOG', 'VATALIS KI 2006 ENVIRON MANAGE'), ('XIAO HY 2011 SOIL SEDIMENT CONTAM', 'VATALIS KI 2006 ENVIRON MANAGE'), ('ANGIULLI F 2005 IEEE T KNOWL DATA EN', 'CHAKRABORTI S 2001 J QUAL TECHNOL')])

        self.assertSetEqual(obtained, expected, "Edges not as expected")

        expected = set(['YU HL 2007 STOCH ENV RES RISK A', 'ZOU CK 2011 TECHNOMETRICS', 'FILIPPIDIS A 1992 FUEL', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN', 'LANG CL 2012 NAV RES LOG', 'EFRON B 1993 INTRO BOOTSTRAP', 'RAO JL 1992 RNEA TECHN B SER LAN', 'DAS N 2009 QUAL TECHNOL QUANT M', 'FRANK A 2010 UCI MACHINE LEARNING', 'XIAO HY 2011 SOIL SEDIMENT CONTAM', 'VATALIS KI 2006 ENVIRON MANAGE', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE', 'CHAKRABORTI S 2001 J QUAL TECHNOL'])
        obtained = set(self.cocitations_zero.nodes())
        self.assertSetEqual(obtained, expected, "Nodes not as expected")


    def test_cocitations_one(self):
        self.assertEqual(nx.number_of_nodes(self.cocitations_one), 13)
        self.assertEqual(nx.number_of_edges(self.cocitations_one), 58)

        obtained = set(self.cocitations_one.edges())
        expected = set([('YU HL 2007 STOCH ENV RES RISK A', 'ZOU CK 2011 TECHNOMETRICS'), ('YU HL 2007 STOCH ENV RES RISK A', 'FILIPPIDIS A 1992 FUEL'), ('YU HL 2007 STOCH ENV RES RISK A', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('YU HL 2007 STOCH ENV RES RISK A', 'LANG CL 2012 NAV RES LOG'), ('YU HL 2007 STOCH ENV RES RISK A', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('YU HL 2007 STOCH ENV RES RISK A', 'VATALIS KI 2006 ENVIRON MANAGE'), ('YU HL 2007 STOCH ENV RES RISK A', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE'), ('ZOU CK 2011 TECHNOMETRICS', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('ZOU CK 2011 TECHNOMETRICS', 'FILIPPIDIS A 1992 FUEL'), ('ZOU CK 2011 TECHNOMETRICS', 'LANG CL 2012 NAV RES LOG'), ('ZOU CK 2011 TECHNOMETRICS', 'EFRON B 1993 INTRO BOOTSTRAP'), ('ZOU CK 2011 TECHNOMETRICS', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('ZOU CK 2011 TECHNOMETRICS', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE'), ('ZOU CK 2011 TECHNOMETRICS', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('ZOU CK 2011 TECHNOMETRICS', 'FRANK A 2010 UCI MACHINE LEARNING'), ('ZOU CK 2011 TECHNOMETRICS', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('ZOU CK 2011 TECHNOMETRICS', 'VATALIS KI 2006 ENVIRON MANAGE'), ('ZOU CK 2011 TECHNOMETRICS', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('FILIPPIDIS A 1992 FUEL', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('FILIPPIDIS A 1992 FUEL', 'LANG CL 2012 NAV RES LOG'), ('FILIPPIDIS A 1992 FUEL', 'EFRON B 1993 INTRO BOOTSTRAP'), ('FILIPPIDIS A 1992 FUEL', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('FILIPPIDIS A 1992 FUEL', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE'), ('FILIPPIDIS A 1992 FUEL', 'FRANK A 2010 UCI MACHINE LEARNING'), ('FILIPPIDIS A 1992 FUEL', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('FILIPPIDIS A 1992 FUEL', 'VATALIS KI 2006 ENVIRON MANAGE'), ('FILIPPIDIS A 1992 FUEL', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('FILIPPIDIS A 1992 FUEL', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('FRANK A 2010 UCI MACHINE LEARNING', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('FRANK A 2010 UCI MACHINE LEARNING', 'EFRON B 1993 INTRO BOOTSTRAP'), ('FRANK A 2010 UCI MACHINE LEARNING', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('FRANK A 2010 UCI MACHINE LEARNING', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('FRANK A 2010 UCI MACHINE LEARNING', 'LANG CL 2012 NAV RES LOG'), ('FRANK A 2010 UCI MACHINE LEARNING', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('EFRON B 1993 INTRO BOOTSTRAP', 'RAO JL 1992 RNEA TECHN B SER LAN'), ('EFRON B 1993 INTRO BOOTSTRAP', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('EFRON B 1993 INTRO BOOTSTRAP', 'LANG CL 2012 NAV RES LOG'), ('EFRON B 1993 INTRO BOOTSTRAP', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('EFRON B 1993 INTRO BOOTSTRAP', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'DAS N 2009 QUAL TECHNOL QUANT M'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'LANG CL 2012 NAV RES LOG'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'VATALIS KI 2006 ENVIRON MANAGE'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('ADRIANO DC 2001 TRACE ELEMENTS TERRE', 'LANG CL 2012 NAV RES LOG'), ('ADRIANO DC 2001 TRACE ELEMENTS TERRE', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('ADRIANO DC 2001 TRACE ELEMENTS TERRE', 'VATALIS KI 2006 ENVIRON MANAGE'), ('DAS N 2009 QUAL TECHNOL QUANT M', 'LANG CL 2012 NAV RES LOG'), ('DAS N 2009 QUAL TECHNOL QUANT M', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('DAS N 2009 QUAL TECHNOL QUANT M', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('LANG CL 2012 NAV RES LOG', 'CHAKRABORTI S 2001 J QUAL TECHNOL'), ('LANG CL 2012 NAV RES LOG', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN'), ('LANG CL 2012 NAV RES LOG', 'XIAO HY 2011 SOIL SEDIMENT CONTAM'), ('LANG CL 2012 NAV RES LOG', 'VATALIS KI 2006 ENVIRON MANAGE'), ('XIAO HY 2011 SOIL SEDIMENT CONTAM', 'VATALIS KI 2006 ENVIRON MANAGE'), ('ANGIULLI F 2005 IEEE T KNOWL DATA EN', 'CHAKRABORTI S 2001 J QUAL TECHNOL')])

        self.assertSetEqual(obtained, expected, "Edges not as expected")

        expected = set(['YU HL 2007 STOCH ENV RES RISK A', 'ZOU CK 2011 TECHNOMETRICS', 'FILIPPIDIS A 1992 FUEL', 'FRANK A 2010 UCI MACHINE LEARNING', 'EFRON B 1993 INTRO BOOTSTRAP', 'RAO JL 1992 RNEA TECHN B SER LAN', 'ADRIANO DC 2001 TRACE ELEMENTS TERRE', 'DAS N 2009 QUAL TECHNOL QUANT M', 'LANG CL 2012 NAV RES LOG', 'XIAO HY 2011 SOIL SEDIMENT CONTAM', 'VATALIS KI 2006 ENVIRON MANAGE', 'ANGIULLI F 2005 IEEE T KNOWL DATA EN', 'CHAKRABORTI S 2001 J QUAL TECHNOL'])
        obtained = set(self.cocitations_one.nodes())

        self.assertSetEqual(obtained, expected, "Nodes not as expected")


    def test_cocitations_two(self):
        self.assertEqual(nx.number_of_nodes(self.cocitations_two),4)
        self.assertEqual(self.cocitations_two.number_of_edges(), 6)


        obtained = set(self.cocitations_two.edges())
        expected = set([('LANG CL 2012 NAV RES LOG', 'ZOU CK 2011 TECHNOMETRICS'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'ZOU CK 2011 TECHNOMETRICS'), ('ZOU CK 2011 TECHNOMETRICS', 'FILIPPIDIS A 1992 FUEL'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'LANG CL 2012 NAV RES LOG'), ('LANG CL 2012 NAV RES LOG', 'FILIPPIDIS A 1992 FUEL'), ('RAO JL 1992 RNEA TECHN B SER LAN', 'FILIPPIDIS A 1992 FUEL')])

        self.assertSetEqual(obtained, expected, "Edges not as expected")

        expected = set(['RAO JL 1992 RNEA TECHN B SER LAN', 'FILIPPIDIS A 1992 FUEL', 'ZOU CK 2011 TECHNOMETRICS', 'LANG CL 2012 NAV RES LOG'])
        obtained = set(self.cocitations_two.nodes())
        self.assertSetEqual(obtained, expected, "Nodes not as expected")

    def tearDown(self):
         pass



class TestTopCitedParameters(unittest.TestCase):
    """
        Test all the types of network
        Presently for paper_cocitations.
        Assumes reader is functioning

        "../testsuite/testin/paper_cocitations#62809724.txt"
        file has been constructed with
        the following properties:

        There are 2 papers , 3 references are cited 2 times.
        ANGIULLI F 2005 IEEE T KNOWL DATA EN
        RAO J 1992 RNEA TECHN B SER LAN
        FILIPPIDIS A 1992 FUEL

        """

    def setUp(self):
        """ Setting up"""

        wos_data = \
         rd.wos.parse("../testsuite/testin/paper_cocitations#62809724.txt")
        meta_list = rd.wos.convert(wos_data)

        # Trying with no function arguments as there are default ones.
        self.top_cited,self.citation_count = nt.papers.top_cited(meta_list)

        self.top_parents,self.top_cited,self.citation_count = \
                                            nt.papers.top_parents(meta_list)

        self.citation_count = nt.papers.citation_count(meta_list)

        # Trying with different function argument other than default one.
        self.top_cited_date,self.citation_count_date = \
                                 nt.papers.top_cited(meta_list,10)

        self.top_parents_date,self.top_cited_date,self.citation_count_date  = \
                        nt.papers.top_parents(meta_list,12)

        self.citation_count_date = nt.papers.citation_count(meta_list,'date')

        # TypeError slice indices must be integers or None or \
        #  have an __index__ method
        # This error will be encountered if a wrong value is given in \
        #  top_parents and top_cited functions



    def test_top_cited(self):
        obtained = set(self.top_cited)
        expected = set(['YU HL 2007 STOCH ENV RES RISK A',
                         'CHAKRABORTI S 2001 J QUAL TECHNOL',
                         'FILIPPIDIS A 1992 FUEL', \
                         'FRANK A 2010 UCI MACHINE LEARNING',
                         'EFRON B 1993 INTRO BOOTSTRAP',
                         'RAO JL 1992 RNEA TECHN B SER LAN',
                         'ADRIANO DC 2001 TRACE ELEMENTS TERRE',
                         'DAS N 2009 QUAL TECHNOL QUANT M',
                         'LANG CL 2012 NAV RES LOG',
                         'XIAO HY 2011 SOIL SEDIMENT CONTAM',
                         'VATALIS KI 2006 ENVIRON MANAGE',
                         'ANGIULLI F 2005 IEEE T KNOWL DATA EN',
                         'ZOU CK 2011 TECHNOMETRICS'])
        self.assertSetEqual(obtained, expected, "Edges not as expected")

    def test_top_cited_date(self):
        obtained = set(self.top_cited_date)
        expectedt = set(['YU HL 2007 STOCH ENV RES RISK A',
                         'CHAKRABORTI S 2001 J QUAL TECHNOL',
                         'FILIPPIDIS A 1992 FUEL', \
                         'FRANK A 2010 UCI MACHINE LEARNING',
                         'EFRON B 1993 INTRO BOOTSTRAP',
                         'RAO JL 1992 RNEA TECHN B SER LAN',
                         'ADRIANO DC 2001 TRACE ELEMENTS TERRE',
                         'DAS N 2009 QUAL TECHNOL QUANT M',
                         'LANG CL 2012 NAV RES LOG',
                         'XIAO HY 2011 SOIL SEDIMENT CONTAM',
                         'VATALIS KI 2006 ENVIRON MANAGE',
                         'ANGIULLI F 2005 IEEE T KNOWL DATA EN',
                         'ZOU CK 2011 TECHNOMETRICS'])
        self.assertSetEqual(obtained, expected, "Edges not as expected")


    def test_top_parents(self):
        obtained_list = []
        for p in self.top_parents:
            obtained_list.append(p['ayjid'])
        expected_list = ['MODIS K 2014 ', 'TUERHONG G 2014 ']
        self.assertListEqual(obtained_list, \
                                expected_list, "Edges List is not as expected")
        pass
    def test_top_cited_date(self):
        obtained_list = []
        for p in self.top_parents:
            obtained_list.append(p['ayjid'])
        expected_list = ['MODIS K 2014 ', 'TUERHONG G 2014 ']
        self.assertListEqual(obtained_list, \
                             expected_list, "Edges List is not as expected")

    def test_citation_count(self):
        expected = {'YU HL 2007 STOCH ENV RES RISK A': 1, 'CHAKRABORTI S 2001 J QUAL TECHNOL': 1, 'FILIPPIDIS A 1992 FUEL': 2, 'LANG CL 2012 NAV RES LOG': 1, 'EFRON B 1993 INTRO BOOTSTRAP': 1, 'RAO JL 1992 RNEA TECHN B SER LAN': 2, 'ANGIULLI F 2005 IEEE T KNOWL DATA EN': 2, 'DAS N 2009 QUAL TECHNOL QUANT M': 1, 'FRANK A 2010 UCI MACHINE LEARNING': 1, 'XIAO HY 2011 SOIL SEDIMENT CONTAM': 1, 'VATALIS KI 2006 ENVIRON MANAGE': 1, 'ADRIANO DC 2001 TRACE ELEMENTS TERRE': 1, 'ZOU CK 2011 TECHNOMETRICS': 1}
        obtained = self.citation_count
        self.assertDictEqual(expected, obtained, "Edge attribs not as expected")

    def test_citation_count_date(self):
        expected_citations_count_dict = \
            {1992: 4, 1993: 1, 2001: 2, 2005: 2, 2006: 1, 2007: 1, 2009: 1, \
                2010: 1, 2011: 2, 2012: 1}
        obtained_citations_count_dict = self.citation_count_date
        self.assertDictEqual(expected_citations_count_dict, \
                             obtained_citations_count_dict,\
                             "Edge Attribs are equal")

#Custom Error Defined
class NetworkXError(Exception):
    pass

if __name__ == '__main__':
    unittest.main()



import unittest
import tethne.data as ds
import tethne.analyze.collection as co
import networkx as nx
import numpy as np

class TestCollection(unittest.TestCase):
    """
    Tests methods for tethne.analyze.collection module.
    """
    def setUp(self):
        self.collection = ds.GraphCollection()
        self.N = 100    # Number of nodes
        self.I = 5  # Number of graph indices.
        for graph_index in xrange(self.I):
            d = np.random.random((self.N, self.N))
            g = nx.Graph()
            for i in xrange(self.N):
                for j in xrange(i+1, self.N):
                    if d[i, j] > 0.7:
                        g.add_edge(i, j, index=graph_index)
                g.node[i]['graph_index'] = graph_index
            self.collection.graphs[graph_index] = g
        self.test_graph = nx.Graph()

    def test_algorithm(self):
        """
        Test of tethne.analyze.collection.algorithm method.
        """

        # Properly-formed request, with kwarg.
        results = co.algorithm(self.collection, 'betweenness_centrality', k=None)

        # Correct number of nodes in results.
        self.assertEqual(len(results.keys()), self.N)

        # Correct number of graph indices in result for first node.
        self.assertEqual(len(results.values()[0].keys()), self.I)

        # Poorly-formed request, where no such name exists.
        with self.assertRaises(ValueError):
            co.algorithm(self.collection, 'asdf')

        # Poorly-formed request, where name is not a function.
        with self.assertRaises(ValueError):
            co.algorithm(self.collection, 'connected')

    def test_connected(self):
        """
        Test of tethne.analyze.collection.connected method.
        """

        # Properly-formed request.
        results = co.connected(self.collection, 'is_connected')

        # Correct number of graph indices in result.
        self.assertEqual(len(results.keys()), self.I )

        # Poorly-formed request, where no such name exists.
        with self.assertRaises(ValueError):
            co.connected(self.collection, 'asdf')

    def test_edge_history(self):
        """
        Tests for tethne.analyze.collection.edge_history().
        """

        # Value of edge attribute graph_index should match Graph key.
        graph_indices = co.edge_history(self.collection, 0, 1, 'graph_index')
        self.assertEqual(graph_indices.keys(), graph_indices.values())

    def test_node_history(self):
        """
        Tests for tethne.analyze.collection.node_history().
        """

        # Value of node attribute graph_index should match Graph key.
        graph_indices = co.node_history(self.collection, 0, 'graph_index')
        self.assertEqual(graph_indices.keys(), graph_indices.values())

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



"""
Methods for generating Numpy data objects from JSTOR Data-for-Research datasets.

.. autosummary::

   array
   matrix

"""

import numpy as np

class Map(object):
    """
    Maps integer indices to string values.
    """

    def __init__(self):
        self.by_str = {}
        self.by_int = {}

    def __setitem__(self, key, value):
        if type(key) == str:
            self.by_str[key] = value
            self.by_int[value] = key
        if type(key) == int:
            self.by_int[key] = value
            self.by_str[value] = key

    def __getitem__(self, key):
        if type(key) == str:
            return self.by_str[key]
        if type(key) == int:
            return self.by_int[key]

    def __len__(self):
        return len(self.by_str)

def array(data, normalize=False, verbose=False):
    """
    Yields a Numpy array, along with feature-index and document-index mappings.

    **Usage**

    .. code-block:: python

       >>> import tethne.readers as rd
       >>> import tethne.matrices as mt
       >>> data = rd.dfr.ngrams("/Path/to/DfR/data")
       >>> A, doc_index, feat_index = mt.dfr.array(data, normalize=True)

    Parameters
    ----------
    data : dict
        Keys are document identifiers (e.g. DOIs), values are lists of feature-
        frequency tuples.
    normalize : bool
        If True, matrix values are relative to the maximum value in the matrix.

    Returns
    -------
    A : Numpy array
        Columns are documents, rows are features.
    document_index : class:`.Map`
        Maps column indices to document identifiers (keys of provided data).
    feature_index : :class:`.Map`
        Maps row indices to features.

    """
    document_index, feature_index = _index_data(data, verbose)
    N_docs = len(document_index)
    N_feat = len(feature_index)
    max_v = np.max([ np.float(v[1]) for doc in data.keys() for v in data[doc] ])

    if verbose:
        print "array(): {0} documents, {1} features".format(N_docs, N_feat)

    if normalize:
        dtype = np.float
    else:
        dtype = np.int

    A = np.zeros((N_docs, N_feat), dtype=dtype)
    d = 0

    for doc, features in data.iteritems():
        if verbose:
            d += 1
            if d%200 == 0:
                print "array(): Processed {0} of {1} documents"\
                                                              .format(d, N_docs)

        i = document_index[doc]
        for f, v in features:
            j = feature_index[f]
            if normalize:
                v = v/max_v
            A[i, j] = v

    if verbose:
        print "array(): Processed all documents."

    return A, document_index, feature_index

def matrix(data, normalize=False, verbose=False):
    """
    Yields a Numpy matrix, along with feature-index and document-index mappings.

    **Usage**

    .. code-block:: python

       >>> import tethne.readers as rd
       >>> import tethne.matrices as mt
       >>> data = rd.dfr.ngrams("/Path/to/DfR/data")
       >>> M, doc_index, feat_index = mt.dfr.matrix(data, normalize=True)

    Parameters
    ----------
    data : dict
        Keys are document identifiers (e.g. DOIs), values are lists of feature-
        frequency tuples.
    normalize : bool
        If True, matrix values are relative to the maximum value in the matrix.

    Returns
    -------
    M : Numpy matrix
        Columns are documents, rows are features.
    document_index : class:`.Map`
        Maps column indices to document identifiers (keys of provided data).
    feature_index : :class:`.Map`
        Maps row indices to features.

    """

    if verbose:
        print "matrix(): converting array to matrix."

    A, document_index, feature_index = array(data, normalize, verbose)
    M = np.asmatrix(A)

    if verbose:
        print "matrix(): done."

    return M, document_index, feature_index

def _index_data(data, verbose=False):
    """
    Yields document and feature indices from a data dict.

    Parameters
    ----------
    data : dict
        Keys are document identifiers (e.g. DOIs), values are lists of feature-
        frequency tuples.

    Returns
    -------
    document_index : class:`.Map`
        Maps integer indices to document identifiers (keys of provided data).
    feature_index : :class:`.Map`
        Maps integer row indices to features.
    """

    document_index = Map()
    feature_index = Map()
    N_docs = len(data)

    if verbose:
        print "_index_data(): Indexing {0} documents".format(N_docs)

    features = set()
    for i in xrange(N_docs): # Index documents.
        if verbose and i%200 == 0:
            print "_index_data(): Indexed {0} of {1} documents"\
                                                              .format(i, N_docs)

        document_index[i] = data.keys()[i]
        for f,c in data.values()[i]:
            features.add(f) # Build a set of features.

    N_feat = len(features)
    if verbose:
        print "_index_data(): Done indexing documents."
        print "_index_data(): Indexing {0} features.".format(N_feat)

    features = list(features)
    for i in xrange(N_feat):  # Index features.
        if verbose and i %10000 == 0:
            print "_index_data(): Indexed {0} of {1} features."\
                                                              .format(i, N_feat)

        feature_index[i] = features[i]

    if verbose:
        print "_index_data(): Done indexing features."

    return document_index, feature_index


"""
Methods for network analysis.
"""

from tethne.data import DataCollection, GraphCollection
from tethne.builders import authorCollectionBuilder
import tethne.analyze as az

# [#60463184]
def closeness_introgression(papers, node, window_size, normalize = False):
    """
    Analyzes the global closeness centrality of a node over time.

    Parameters
    ----------
    papers : list
        A list of :class:`.Paper` instances.
    node : any
        Handle of the node to analyze.
    window_size : int
        Size of time-window.
    normalize : bool
        If True, normalizes global closeness centrality for each year against
        the average closeness centrality for that year. This will require
        substantially more processing time, and values will usually be >> 0.

    Returns
    -------
    trajectory : dict
        Global closeness centrality for node over specified period.

    """

    D = DataCollection(papers)
    D.slice('date', 'time_window', window_size=window_size)
    builder = authorCollectionBuilder(D)
    C = builder.build('date', 'coauthors')
    trajectory = az.collection.node_global_closeness_centrality(C, node)
    
    return trajectory